#!/usr/bin/perl -w
#-*- mode: Cperl -*-
#
# Use 'which perl' to verify that the path in the first line of this
# file is correct.
#
######################################################################
#
# This program logs and edits QSOs stored in an SQLite database.
#
######################################################################
#
# Licence:
#
# 'qle.pl' is an experimental QSO logger and log editor for amateur
# radio operators.
# Copyright (C) 2004, 2007, 2008  Wilbert Knol ZL2BSJ
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA
#
######################################################################
#
# See the script makelog.sql for details.
#
######################################################################
#
# Early init:
#
{
    use Tk;
    use warnings;
    use strict;
    use English;
    use Tk::ErrorDialog;
    use Tk::After;

    my $progName = $0;
    use File::Basename;
    my $sprogName = basename($progName);
    $progName =~ s/^\.\///;    # Zap leading './'

    my $configFile = "/etc/qle/qle.conf";
#    my $configFile = $progName;
#    $configFile =~ s/.pl$//;
#    $configFile = "$configFile.conf";

    my $copyRight = "

$progName -  Copyright (C) 2004 - 2007 Wilbert Knol (ZL2BSJ)

This program comes with ABSOLUTELY NO WARRANTY; check the source tree
for details.

This is free software, and you are welcome to redistribute it under
the terms of the GNU General Public License v.2 or later.

In the file:

 $configFile

change 'showCopyright' to '0' to disable this dialog, or execute with
the '--showCopyright=0' command-line switch.

";

    # Read configuration file:

    my $noConfig = "Could not read config file \'$configFile'";

    print "Using config file: $configFile...\n";
    use Config::General;
    my $conf = new Config::General(
        -ConfigFile      => "$configFile",
        -InterPolateVars => 1,

   # Leave out; older versions of Config::General don't know abt it:
   # -InterPolateEnv  => 0,
    ) or die "$noConfig: $!\n";

    my %config = $conf->getall
      or die "$noConfig: $!\n";

  # Override config-file values with command-line values if present:
    foreach my $k ( keys(%config) ) {
        foreach my $a (@ARGV) {
            if ( $a =~ m/--$k=(.+)/ ) {
                $config{$k} = $1;
            }
        }
    }

    # Show command-line args if debug
    if ( $config{debug} ) {
        print "Debugging enabled.\n";
        print "To turn off debugging, change (in file $configFile)";
        print " the line:\n";
        print "  debug = 1\n";
        print "to:\n";
        print "  debug = 0\n";
        print "or execute with command-line switch '--debug=0'\n";
        print "\@ARGV = \'@ARGV\'\n";
    }

    # Create main window.
    my $window = MainWindow->new();

    # Create 2nd window.
    my $tLevel = $window->Toplevel;

    # Check xterm-style command-line switches:
    #    require Tk::CmdLine;
    #    $window->CmdLine;

    $window->packPropagate(0);

    # Configure main window:
    $window->configure(
        -title =>
          "$progName  - QSO Logger and log Editor - Main Window",
        -foreground => $config{fg},
        -background => $config{bg},
    );

    if ( defined $config{mainWindowGeometry} ) {
        $window->geometry( $config{mainWindowGeometry} );
    }

    my $mostRecentSerial =
      "999";    # Used to store most-recent transmitted
                # serial.
        # Paint & populate log-viewer frame in top-half of window:
    my $searchButton = log_viewer( $window, \%config, $progName,
        \$mostRecentSerial );

    # Paint & populate 'new QSO' frame at bottom-half of window:
    my $nqEntryRef = new_qso( $tLevel, $searchButton, \%config,
        \$mostRecentSerial );

    $tLevel->configure( -title =>
          "$progName  - QSO Logger and log Editor - Sub-window", );

    $tLevel->geometry( $config{subWindowGeometry} )
      if defined $config{subWindowGeometry};

    # Plaster GPL info across the screen::
    my $crWindow;

    if ( $config{showCopyright} ) {
        $crWindow = show_copyright( \%config, $window, $copyRight );
        $crWindow->grab;
        $crWindow->raise;
        $crWindow->focus;
        $crWindow->waitWindow;
    }

    set_focus( \%config, $nqEntryRef ) if defined $nqEntryRef;

    # Lurk in the background waiting for the user to Do Something:
    MainLoop;
}

######################################################################
#
# End of main program
#
######################################################################

######################################################################
#
# log_viewer
#
# Purpose:  Populates the top-half of the main window, showing a view
#           of the log, and puts a bunch of action buttons around it.
#
# Args:     1. Instance of a window to display the log viewer in.
#           2. Reference to associative array containing config data.
#           3. Scalar containing name of program.
#           4. Reference to scalar used to store most-recent serial
#              number in
#
# Returns:  1. Search/refresh button.
#
######################################################################
#
sub log_viewer {

    my ( $window, $hashRef, $progName, $mostRecentSerialRef ) = @_;

    my %config = %{$hashRef};

    # Divide up screen into horizontal frames:
    require Tk::LabFrame;
    my $swFrame = $window->LabFrame(
        -relief      => 'raised',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    )->pack( -expand => 1, -fill => 'both' );

    $swFrame->configure(
        -label     => 'log viewer/editor',
        -labelside => 'acrosstop',
    );

    my $swFrame1 = $swFrame->Frame(
        -relief      => 'flat',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    )->pack( -expand => 1, -fill => 'both' );

    my $swFrame2 = $swFrame->Frame(
        -relief      => 'flat',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    )->pack( -expand => 1, -fill => 'both' );

    my $swFrame5 = $swFrame->Frame(
        -relief      => 'flat',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    )->pack( -expand => 1, -fill => 'both' );

    my $swFrame6 = $swFrame->Frame(
        -relief      => 'flat',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    )->pack( -expand => 1, -fill => 'both' );

    # Frame a status line, below hList:
    my (
        $statFrame_l, $statFrame_ml, $statFrame_mr,
        $statFrame_r, $dxccFrame
    ) = stat_frames( \%config, $swFrame );

    my $wantsRecent = 1;
    my $rbString    = $swFrame1->Radiobutton(
        -text             => "search by string",
        -variable         => \$wantsRecent,
        -value            => 0,
        -foreground       => $config{fg},
        -background       => $config{bg},
        -activebackground => $config{bg},
      )->pack(
        -side   => 'left',
        -expand => 1,
        -anchor => 'e'
      );

    $swFrame1->Label(
        -text       => " or  ",
        -foreground => $config{fg},
        -background => $config{bg},
      )->pack(
        -side   => 'left',
        -expand => 0,
        -anchor => 'e'
      );

    my $rbRecent = $swFrame1->Radiobutton(
        -text             => "show most-recent ",
        -variable         => \$wantsRecent,
        -value            => 1,
        -foreground       => $config{fg},
        -background       => $config{bg},
        -activebackground => $config{bg},
      )->pack(
        -side   => 'left',
        -expand => 0,
        -anchor => 'w'
      );

    # Number of QSOs to which we want to limit a log search:
    my $noqEntry = $swFrame1->Entry(
        -validate   => 'key',
        -foreground => $config{entry_fg},
        -background => $config{entry_bg},
        -state      => 'normal'
      )->pack(
        -side   => 'left',
        -anchor => 'w',
      );

    $noqEntry->configure( -textvariable => \$config{rNumber} );

    # Enforce numbers only:
    $noqEntry->configure(
        -validatecommand => sub { $_[1] =~ /[0-9]/; } );

    $swFrame1->Label(
        -text       => " QSOs",
        -foreground => $config{fg},
        -background => $config{bg},
      )->pack(
        -side   => 'left',
        -anchor => 'w',
        -expand => 1,
      );

    my $counterLabel = $statFrame_l->Label(
        -borderwidth => $config{borderwidth},
        -relief      => 'groove',
        -foreground  => $config{fg},
        -background  => $config{bg},
      )->pack(
        -side   => 'left',
        -anchor => 'w',
        -expand => '1',
        -fill   => 'x',
      );

    my $statusTextDb = "db: " . $config{db};
    my $statusLabelb = $statFrame_ml->Label(
        -text        => "$statusTextDb",
        -borderwidth => $config{borderwidth},
        -relief      => 'groove',
        -foreground  => $config{fg},
        -background  => $config{bg},
      )->pack(
        -expand => 1,
        -anchor => 's',
        -fill   => 'x',
      );

    my $statusTextTable = "table: " . $config{tableName};
    my $statusLabelc    = $statFrame_mr->Label(
        -text        => "$statusTextTable",
        -borderwidth => $config{borderwidth},
        -relief      => 'groove',
        -foreground  => $config{fg},
        -background  => $config{bg},
      )->pack(
        -expand => 1,
        -anchor => 's',
        -fill   => 'x',
      );

    my $statusTextUser = "user: ";
    $statusTextUser = $statusTextUser . "$ENV{USER}"
      unless $OSNAME =~ /win/i;

    my $statusLabeld = $statFrame_r->Label(
        -text        => "$statusTextUser",
        -borderwidth => $config{borderwidth},
        -relief      => 'groove',
        -foreground  => $config{fg},
        -background  => $config{bg},
      )->pack(
        -expand => 1,
        -anchor => 's',
        -fill   => 'x',
        -side   => 'left',
      );

    my $statusTextUid = "UID: " . "$UID";
    my $statusLabele  = $statFrame_r->Label(
        -text        => "$statusTextUid",
        -borderwidth => $config{borderwidth},
        -relief      => 'groove',
        -foreground  => $config{fg},
        -background  => $config{bg},
      )->pack(
        -expand => 1,
        -anchor => 's',
        -fill   => 'x',
        -side   => 'right',
      );

    my $sfsLabel = $swFrame2->Label(
        -text       => "Search for string: ",
        -foreground => $config{fg},
        -background => $config{bg},
      )->pack(
        -side   => 'left',
        -anchor => 'e',
      );

    my $entry = $swFrame2->Entry->pack(
        -side   => 'left',
        -anchor => 'w',
    );

    $swFrame2->Label(
        -text       => "    in field: ",
        -foreground => $config{fg},
        -background => $config{bg},
      )->pack(
        -side   => 'left',
        -anchor => 'e',
      );

    my $sbOptionmenu = $swFrame2->Optionmenu(
        -foreground => $config{fg},
        -background => $config{bg},
      )->pack(
        -side   => 'left',
        -anchor => 'w',
      );

    $swFrame2->Label(
        -text       => "    using a ",
        -foreground => $config{fg},
        -background => $config{bg},
      )->pack(
        -side   => 'left',
        -anchor => 'e',
      );

    my $matchType       = "";
    my $s               = "strict, case-sensitive match";
    my $l               = "loose match, allowing wild-cards";
    my $matchOptionmenu = $swFrame2->Optionmenu(
        -foreground       => $config{fg},
        -background       => $config{bg},
        -activebackground => $config{bg},

        # -underline        => '0',
      )->pack(
        -expand => '1',
        -side   => 'left',
        -anchor => 'w',
      );

    $matchOptionmenu->addOptions( [ "$s" => 'strict' ] );
    $matchOptionmenu->addOptions( [ "$l" => 'loose' ] );
    $matchOptionmenu->configure( -variable => \$matchType );

    $config{defaultMatch} = 'strict'
      unless $config{defaultMatch} eq 'loose';
    $matchType = $config{defaultMatch};

    my $searchButton = $swFrame5->Button->pack(
        -fill   => 'x',
        -side   => 'left',
        -expand => 1,
    );
    my $deleteButton = $swFrame5->Button->pack(
        -fill   => 'x',
        -side   => 'left',
        -expand => 1,
    );
    my $editButton = $swFrame5->Button->pack(
        -fill   => 'x',
        -side   => 'left',
        -expand => 1,
    );

    my $exportMenubutton = $swFrame5->Menubutton->pack(
        -fill   => 'x',
        -side   => 'left',
        -expand => 1,
    );

    # Draw up the list of QSOs found in $hList:
    require Tk::HList;
    my @fieldDescr     = @{ $config{fieldDescr} };
    my $numberOfFields = 0;
    my $v              = 0;
    foreach my $x ( @{ $config{logShowFields} } ) {
        $numberOfFields++ if ${ $config{logShowFields} }[$v] == 1;
        $v++;
    }
    my $hList = $swFrame6->Scrolled(
        'HList',
        -height           => $config{hList_height},
        -scrollbars       => 'osoe',
        -foreground       => $config{hList_fg},
        -background       => $config{hList_bg},
        -selectforeground => $config{hList_sel_fg},
        -selectbackground => $config{hList_sel_bg},
        -columns          => $numberOfFields,
        -header           => 1,
        -itemtype         => 'text',
        -selectmode       => 'multiple',
      )->pack(
        -expand => 1,
        -fill   => 'both',
        -anchor => 's',
      );
    $hList->Subwidget("yscrollbar")->configure(
        -activebackground => $config{sb_activebackground},
        -background       => $config{sb_background},
    );
    $hList->Subwidget("xscrollbar")->configure(
        -activebackground => $config{sb_activebackground},
        -background       => $config{sb_background},
    );

    # Put headers on top:
    my $i = 0;
    my $a = 0;
    foreach my $hdr (@fieldDescr) {
        if ( ${ $config{logShowFields} }[$a] == 1 ) {
            $hList->headerCreate(
                $i,
                -text             => $hdr,
                -headerbackground => $config{hList_hdr_bg},
            );
            $i++;
        }
        $a++;
    }

    # Add a rowID column at the end of the headers:
    $hList->headerCreate(
        $i,
        -text             => "row-ID",
        -headerbackground => $config{rowid_bg},
    );

 # Build 'search-by-<field> option-menu (for field-specific search):
 #
    my @fieldNames = @{ $config{fieldNames} };
    my $sf         = "";

    #
    for ( my $i = 0 ; $i <= $#fieldNames ; $i++ ) {
        my $fn = $fieldNames[$i];
        $sbOptionmenu->addOptions( [ $fieldDescr[$i] => $fn ] );
        if ( $fieldNames[$i] eq $config{defaultSearchField} ) {
            $sf = $fieldDescr[$i];
        }
    }

   # Set textvariable on $sbOptionmenu.
   # Must be done separately to avoid Perl/Tk 800.027-4 ignoring it:

    my $searchField;
    $sbOptionmenu->configure( -textvariable => \$searchField );
    $searchField = $sf;

    # Disable entry widget when 'show recent' radio button pushed:
    $rbRecent->configure(
        -command => sub {
            $entry->configure( -state => "disabled" );
            $matchOptionmenu->configure( -state => 'disabled', );
            $searchButton->configure( -state => "normal" );
            $sbOptionmenu->configure( -state => 'disabled' );
        }
    );

  # Enable entry widget when 'search by string' radio button pushed:
    $rbString->configure(
        -command => sub {
            $entry->configure( -state => "normal" );
            $matchOptionmenu->configure( -state => 'normal', );
            $entry->focus;
            $sbOptionmenu->configure( -state => 'normal' );
            if ( $entry->get() eq "" ) {
                $searchButton->configure( -state => "disabled" );
            }
        }
    );

    $counterLabel->configure(
        -text       => " QSOs found: 0",
        -foreground => $config{fg},
        -background => $config{bg},
    );

    $searchButton->configure(
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -underline        => '9',
        -text             => "Search / Refresh log",
    );

    $deleteButton->configure(
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -text             => "Delete QSO",
        -state            => "disabled",
        -underline        => '0',
        -command          => [
            \&delete_qso,  $hList, $window,
            $searchButton, \%config,
        ]
    );

    $editButton->configure(
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -text             => "Edit QSO",
        -state            => "disabled",
        -underline        => '0',
    );

    $exportMenubutton->configure(
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -text             => "Export QSOs",
        -state            => "disabled",
        -underline        => '2',
    );

    $exportMenubutton->command(
        -label   => 'Export to Cabrillo format',
        -command => [
            \&cabexport_qso, \$hList, \$window,
            \%config,        \$progName,
        ]
    );

    $exportMenubutton->command(
        -label => 'Export to ADIF format',
        -command =>
          [ \&adifexport_qso, \$hList, \$window, \%config, ]
    );

   #    $exportMenubutton->entryconfigure(2, -state => 'disabled',);

    $entry->configure(
        -validate => 'key',
        -validatecommand =>
          [ \&val_entry, $searchButton, $window, ],
        -takefocus => 1,
    );

 # This array contains the rowID(s) of the QSO(s) currenty selected:
    my @selectedRowID;
    my @oldSelectedRowID;

    #
    # Invoke $searchButton if the ENTER key is used when
    # the entry box has the focus:
    $entry->bind(
        '<KeyPress-Return>' => sub {
            $searchButton->invoke();
        }
    );

# When we select (highlight) a QSO in the hList, we store the rowID in
# array $selectedRowID. This is done to make the selection stick
# between log searches (if possible):
    $hList->bind(
        '<ButtonRelease-1>' => sub {
            @selectedRowID = $hList->selectionGet;

            # Removing a single selection:
            # qle-0.0.6pre13 mod.
            # If we have selected only one row...
            if ( @selectedRowID == 1 ) {

                # ...and we previously selected one row...
                if ( @oldSelectedRowID == 1 ) {

                    # ...and the row selected now is the same one we
                    # selected previously...
                    if (   defined $selectedRowID[0]
                        && defined $oldSelectedRowID[0]
                        && $selectedRowID[0] ==
                        $oldSelectedRowID[0] )
                    {

           # ...THEN we should toggle (turn off) the selection.
           # Why? The user may want to have nothing at all selected,
           # and this is a (crude) way to achieve that.
                        $hList->selectionClear( $selectedRowID[0] );

                 # Remove all traces of present and past selections:
                        pop @selectedRowID;
                        pop @oldSelectedRowID;
                    }
                }
                $oldSelectedRowID[0] = $selectedRowID[0];

                # print "size=1\n";
            }

            # Turn $editButton on if something has been selected:
            if ( defined( $hList->selectionGet ) ) {
                $editButton->configure( -state => 'normal' );
                $deleteButton->configure( -state => 'normal' );
                $exportMenubutton->configure( -state => 'normal' );
            }
            else {
                $editButton->configure( -state => 'disabled' );
                $deleteButton->configure( -state => 'disabled' );
                $exportMenubutton->configure(
                    -state => 'disabled' );
            }

        },
    );

    $searchButton->configure(
        -command => [
            \&select_search_method, \$wantsRecent,
            $editButton,            $deleteButton,
            $entry,                 $hList,
            \$searchField,          $counterLabel,
            \$matchType,            \@selectedRowID,
            $searchButton,          \%config,
            \$window,               $mostRecentSerialRef,
        ],
    );

    # Set default mode by invoking the "search by string" or
    # "show recent" buttons:
    if ( $config{defaultMode} =~ /recent/ ) {
        $rbRecent->invoke();
    }
    else {
        $rbString->invoke();
    }

    # Refresh the QSO list by invoking the $searchButton:
    $searchButton->invoke();

    # Set up some $window-wide key bindings:
    $window->bind(
        '<Alt-r>' => sub {
            $searchButton->invoke();
        }
    );
    $window->bind(
        '<Alt-R>' => sub {
            $searchButton->invoke();
        }
    );

    $window->bind(
        '<Alt-d>' => sub {
            $deleteButton->invoke();
        }
    );
    $window->bind(
        '<Alt-D>' => sub {
            $deleteButton->invoke();
        }
    );

    $window->bind(
        '<Alt-e>' => sub {
            $editButton->invoke();
        }
    );
    $window->bind(
        '<Alt-E>' => sub {
            $editButton->invoke();
        }
    );

    $window->bind(
        '<Alt-p>' => sub {
            $exportMenubutton->invoke();
        }
    );
    $window->bind(
        '<Alt-P>' => sub {
            $exportMenubutton->invoke();
        }
    );

    return ($searchButton);
}

######################################################################
#
# subroutine new_qso
#
# Purpose: Populates the remaining (bottom) half an existing window
#          with a 'new QSO' template. The user can fill in the fields
#          and either log the QSO into the database, or wipe the
#          template.
#
#          Upon logging, the log view in the viewer/editor frame is
#          refreshed, showing the newly added QSO (assuming 'show most
#          recent QSOs' is selected).  The 'new QSO' template is
#          festooned with several checkbuttons, pushbuttons and CW
#          function keys. A 'quit' button terminates the program.
#
# Args:
#          1. Window in which to pop up the 'new QSO' panel.
#          2. Instance of a search-button, passed from the log view
#             frame, needed to force a refresh of the log view,
#             following logging the new QSO - see above.
#          3. Reference to associative array holding configuration
#             scalars.
#          4. Name of program (used in cabrillo dumps)
#          4. Referance to scalar containing the most recently
#             transmitted serial number, or undef if none found.
#
# Returns: void.
#
######################################################################
#
sub new_qso {

    my ( $window, $searchButton, $hashRef, $mostRecentSerialRef ) =
      @_;

    my %config        = %{$hashRef};
    my @capAll        = @{ $config{capAll} };
    my @defaultValues = @{ $config{defaultValues} };
    my @fieldNames    = @{ $config{fieldNames} };
    my @fieldDescr    = @{ $config{fieldDescr} };
    my $typoDialog    = $config{typoDialog};
    my $autoCQ        = 0;

    my @scpArray;
    my $doScp = 0;

    stat( $config{masterScp} );
    if ( -r _ ) {
        $doScp = process_scp( \@scpArray, $config{masterScp} );
    }
    else {
        print
"Can't read super check partial file $config{masterScp}\n",
          "No super check partial.\n";
    }

    my $scpWindow;
    my $scpLabel;
    my $scpString = "";

    if ( $doScp == 1 ) {

        $scpWindow = $window->Toplevel;
        $scpWindow->title("SCP");
        $scpWindow->configure(
            -background => $config{mainWindow_bg},
            -relief     => 'groove',
        );
        $scpWindow->geometry( $config{scpGeometry} );

        $scpLabel = $scpWindow->Label(
            -relief      => 'raised',
            -borderwidth => $config{borderwidth},
            -foreground  => $config{fg},
            -background  => $config{bg},
            -justify     => 'left',
          )->pack(
            -expand => '1',
            -fill   => 'both',
            -side   => 'top',
            -anchor => 'w',
          );

        $scpLabel->configure( -textvariable => \$scpString, );

    }

    my %ctyHash;
    my %contHash;
    my %cqzHash;
    my %ituzHash;
    stat( $config{ctyDat} );
    if ( -r _ ) {
        process_cty_dat( \%ctyHash, \%contHash, \%cqzHash,
            \%ituzHash, $config{ctyDat} );
    }
    else {
        print "Can't read cty.dat file $config{ctyDat}\n";
    }

    # Set up frame, used for entering new QSOs:
    require Tk::LabFrame;
    my $nqFrame = $window->LabFrame(
        -relief      => 'raised',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    )->pack( -expand => '1', -fill => 'x', -side => 'top' );

    $nqFrame->configure(
        -label     => 'New QSO template',
        -labelside => 'acrosstop',
    );

    my $nqFrameB = $nqFrame->Frame(
        -foreground => $config{fg},
        -background => $config{bg},
    )->pack( -expand => '1', -fill => 'x', -side => 'top' );

    my $nqFrameB1 = $nqFrameB->Frame(
        -foreground => $config{fg},
        -background => $config{bg},
    )->pack( -expand => '1', -fill => 'x', -side => 'left' );

    my $nqFrameB2 = $nqFrameB->Frame(
        -foreground => $config{fg},
        -background => $config{bg},
    )->pack( -expand => '1', -fill => 'x', -side => 'left' );

    my $nqFrameB3 = $nqFrameB->Frame(
        -foreground => $config{fg},
        -background => $config{bg},
    )->pack( -expand => '1', -fill => 'x', -side => 'left' );

    my $utcCb = $nqFrameB1->Checkbutton(
        -text             => "Use UTC clock",
        -variable         => \$config{utcClock},
        -foreground       => $config{fg},
        -background       => $config{bg},
        -activebackground => $config{bg},
        -underline        => '0',
      )->pack(
        -expand => '1',
        -side   => 'left',
        -anchor => 's',
      );

    # Check CW keyer as per subroutine config_cwd():
    if ( $config{noCwDaemon} != 1 ) {

        # CW support requested in config file:
        my $retValue = 0;
        ( $retValue, $cwdSocket ) = config_cwd($hashRef);

        if ( $retValue == 1 ) {

            # CW support requested but no daemon found.
            # Show warning in dialog box...
            my $w =
"In the config file, CW support was requested by:\n\n";
            $w = $w . "\'noCwDaemon = 0\'\n\n";
            $w = $w . "...but no CW daemon was found.\n\n";
            $w = $w
              . "Perhaps you need to start the daemon (as user root)";
            $w =
              $w . " with:\n\n\'/etc/init.d/cwdaemon start\'\n\n";
            $w = $w . "Check that the daemon is up with:\n\n";
            $w = $w . "\'ps ax \| grep cwdaemon\'\n\n";
            $w = $w . "For now, CW support is turned off.\n\n";
            print "$w" if $config{debug};
            my_dialog( $hashRef, $window, "Warning", "Close",
                $config{dialogGeometry}, "$w" );
            $config{noCwDaemon} = 1;
            ${$hashRef}{noCwDaemon} = 1;
        }
    }
    else {

        # CW support was disabled in config file:
        print "CW disabled in config file: noCwDaemon=1\n"
          if $config{debug};
    }

    my $abbrevCb = $nqFrameB2->Checkbutton(
        -text             => "Abbreviate serials on CW",
        -variable         => \$config{abbrevNum},
        -foreground       => $config{fg},
        -background       => $config{bg},
        -activebackground => $config{bg},
        -underline        => '1',
      )->pack(
        -expand => '1',
        -side   => 'left',
        -anchor => 's',
      ) unless $config{noCwDaemon};

    my $leadingZerosCb = $nqFrameB3->Checkbutton(
        -text             => "Use serials with Leading Zeros  ",
        -variable         => \$config{leadingZeros},
        -foreground       => $config{fg},
        -background       => $config{bg},
        -activebackground => $config{bg},
        -underline        => '17',
      )->pack(
        -expand => '1',
        -side   => 'left',
        -anchor => 'w',
      );

    $nqFrameB3->Label(
        -text       => 'WPM: ',
        -foreground => $config{fg},
        -background => $config{bg},
      )->pack( -expand => 1, -side => 'left', -anchor => 'e', )
      unless $config{noCwDaemon};

    my $wpmOptionMenu = $nqFrameB3->Optionmenu(
        -textvariable     => \$config{cwdWpm},
        -foreground       => $config{fg},
        -background       => $config{bg},
        -activebackground => $config{bg},
        -underline        => '0',
        -options          => [ @{ $config{cwSpeed} } ],
        -command          => [ \&config_cwd, \%config ],
      )->pack(
        -expand => '1',
        -side   => 'left',
        -anchor => 'w',
      ) unless $config{noCwDaemon};

    # Status label:
    my $statLabel = $nqFrame->Label(
        -text        => '',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    )->pack( -expand => '1', -fill => 'x', -side => 'bottom' );

    my $fkeyFrame = $window->Frame(
        -relief      => 'flat',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
      )->pack(
        -expand => '1',
        -fill   => 'x',

        #        -side   => 'bottom',
        -anchor => 's'
      );

    my $quitFrame = $window->Frame(
        -relief      => 'flat',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
      )->pack(
        -expand => '1',
        -fill   => 'x',

        #        -side   => 'bottom',
        -anchor => 's'
      );

    my $statusFrame = $window->Frame(
        -relief      => 'flat',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
      )->pack(
        -expand => '1',
        -fill   => 'x',
        -side   => 'bottom',
        -anchor => 's'
      );

    # Frame 'new qso template'  status line:
    my (
        $statFrame_l, $statFrame_ml, $statFrame_mr,
        $statFrame_r, $dxccFrame
    ) = stat_frames( \%config, $statusFrame );

    # ..and fill with infoString items from config file:
    my $n           = 0;
    my @infoStrings = @{ $config{infoString} };

    my $dxccStr   = "DXCC info bar";
    my $dxccLabel = $dxccFrame->Label(
        -textvariable => \$dxccStr,
        -borderwidth  => $config{borderwidth},
        -relief       => 'groove',
        -foreground   => $config{fg},
        -background   => $config{bg},
      )->pack(
        -side   => 'left',
        -anchor => 'w',
        -expand => '1',
        -fill   => 'x',
      );

    my @frameArray =
      ( $statFrame_l, $statFrame_ml, $statFrame_mr, $statFrame_r );
  STR: foreach my $iStr (@infoStrings) {
        if ( $n > $#frameArray && $config{debug} ) {
            print
"Too many items called \'infoString\' in config file.\n";
            print "No more than $#frameArray items allowed.\n";
            last STR;
        }

        $frameArray[$n]->Label(
            -text        => $iStr,
            -borderwidth => $config{borderwidth},
            -relief      => 'groove',
            -foreground  => $config{fg},
            -background  => $config{bg},
          )->pack(
            -expand => 1,
            -anchor => 's',
            -fill   => 'x',
          );

        $n++;
    }

    my $quitButton = $quitFrame->Button->pack(
        -fill   => 'x',
        -expand => 1,
        -side   => 'left',
        -anchor => 's',
    );

    $quitButton->configure(
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -underline        => '1',
        -text             => "Exit program",
        -command          => [ \&exit_program, $hashRef, $window ],
    );

    my $nqFrameLeft = $nqFrame->Frame(
        -relief      => 'flat',
        -foreground  => $config{fg},
        -background  => $config{bg},
        -borderwidth => $config{borderwidth},
    );
    $nqFrameLeft->pack(
        -side   => 'left',
        -expand => 1,
        -fill   => 'both',
    );

    my $nqFrameCenter = $nqFrame->Frame(
        -relief      => 'flat',
        -foreground  => $config{fg},
        -background  => $config{bg},
        -borderwidth => $config{borderwidth},
    );

    my $nqFrameRight = $nqFrame->Frame(
        -relief      => 'flat',
        -foreground  => $config{fg},
        -background  => $config{bg},
        -borderwidth => $config{borderwidth},
    );
    $nqFrameRight->pack(
        -side   => 'right',
        -expand => 1,
        -fill   => 'both',
    );
    $nqFrame->pack(
        -side   => 'top',
        -expand => 1,
        -fill   => 'both',
    );
    $nqFrameCenter->pack(
        -side   => 'left',
        -expand => 1,
        -fill   => 'both',
    );

    my @subFrame;
    my @leftSubFrame;
    my @rightSubFrame;
    my @label;
    my @nqEntry;

   # Create sub-frames for each field, containing a header (left)
   # and an Entry box for the field (right).
   # Associative array containing field names and entry widget pairs
   # for a newly entered QSO. E.g: key 'call' is paired with the
   # Entry widget for field 'call'.

    my %nqTable;

    # Figure out a suitable width for the field description labels:
    my $descrWidth = 0;

    # maxDescrWidth
    print "maxDescrWidth= $config{maxDescrWidth}\n"
      if $config{debug};
    if ( defined( $config{maxDescrWidth} ) ) {
        $descrWidth = $config{maxDescrWidth};
    }
    else {
        foreach my $f (@fieldDescr) {
            my $l = length($f);
            $descrWidth = $l if $l > $descrWidth;
        }
    }
    $descrWidth++;
    $descrWidth++;

    # Figure out a suitable size for the field value entry boxes:
    my $maxFieldLength = 0;
    my @fieldLengths   = @{ $config{fieldLengths} };
    my @autoFields;

    if ( defined $config{maxFieldLength} ) {
        $maxFieldLength = $config{maxFieldLength};
    }
    else {
        foreach my $l (@fieldLengths) {
            $maxFieldLength = $l if $l > $maxFieldLength;
        }
    }

    my @typeOms;    # Array of references to 'type' option menus.

    # Array of initial values for field types:
    my @fieldTypes = @{ $config{fieldTypes} }
      or die
      "Could not read array \@fieldTypes from config file: $!\n";

    my @hdrtxt = ( 'FIELD', 'VALUE', 'TYPE' );

    # Print header above left entry boxes:
    my $hdrFrameLeft = $nqFrameLeft->Frame(
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    )->pack( -expand => 1, -fill => 'x', -side => 'top' );

    foreach my $h (@hdrtxt) {
        $hdrFrameLeft->Label(
            -text        => $h,
            -borderwidth => $config{borderwidth},
            -foreground  => $config{fg},
            -background  => $config{bg},
        )->pack( -expand => 1, -fill => 'both', -side => 'left' );
    }

    # Print header above right entry boxes:
    my $hdrFrameRight = $nqFrameRight->Frame(
        -foreground  => $config{fg},
        -background  => $config{bg},
        -borderwidth => $config{borderwidth},
    )->pack( -expand => 1, -fill => 'x', -side => 'top' );

    # print header above right entry boxes:
    foreach my $h (@hdrtxt) {
        $hdrFrameRight->Label(
            -text        => $h,
            -borderwidth => $config{borderwidth},
            -foreground  => $config{fg},
            -background  => $config{bg},
        )->pack( -expand => 1, -fill => 'both', -side => 'left' );
    }

    # Check out rig interface:
    my $rigLock =
      0;    # If '1', the rig is busy, skip this poll until '0'
    my ( $rigctldSocket, $noRig, $mode, $khz ) =
      config_rigctld( $hashRef, $window, \$rigLock, );

    my @tVars;

    # Set up - from left to right:
    # field description labels, entry boxes and type menus:
    for ( my $i = 0 ; $i <= $#fieldNames ; $i++ ) {

        if ( $i < @fieldNames / 2 ) {
            $subFrame[$i] = $nqFrameLeft->Frame(
                -relief      => 'groove',
                -foreground  => $config{fg},
                -background  => $config{bg},
                -borderwidth => $config{borderwidth},
            )->pack( -side => 'top' );
        }
        else {
            $subFrame[$i] = $nqFrameRight->Frame(
                -relief      => 'groove',
                -foreground  => $config{fg},
                -background  => $config{bg},
                -borderwidth => $config{borderwidth},
            )->pack( -side => 'top' );
        }

        # Set up field description labels on the left:
        $label[$i] = $subFrame[$i]->Label(
            -text       => "$fieldDescr[$i]",
            -anchor     => 'w',
            -width      => "$descrWidth",
            -foreground => $config{labels_fg},
            -background => $config{labels_bg},
          )->pack(
            -side   => 'left',
            -expand => 1,
            -fill   => 'both',
          );

        # Set up field entry boxes on the right:

        $nqEntry[$i] = $subFrame[$i]->Entry(
            -width      => "$maxFieldLength",
            -validate   => 'key',
            -foreground => $config{entry_fg},
            -background => $config{entry_bg},
            -state      => 'normal'
        );
        $nqEntry[$i]->configure( -textvariable => \$tVars[$i], );

        $nqEntry[$i]->configure(
            -validatecommand => [
                \&check_entry, \$autoCQ,     \%config,
                $i,            $nqEntry[$i], \$tVars[$i],
            ]
        );

        $tVars[$i] = $defaultValues[$i];
        $nqEntry[$i]->pack(
            -side   => 'left',
            -expand => 1,
            -fill   => 'both',
        );

        # Add 'type' option menu  to field on far right:
        $typeOms[$i] = $subFrame[$i]->Optionmenu(
            -state   => 'normal',
            -options => [ "CAR", "LCK", "---", ],

            # Do not uncomment statement below.
            # Must be done separately.*** Perl/Tk bug*** ?
            # -textvariable => \$fieldTypes[$i],
            -foreground => $config{type_fg},
            -background => $config{type_bg},
            -takefocus  => 0,
          )->pack(
            -side   => 'left',
            -expand => 0,
            -anchor => 'w'
          );

        $typeOms[$i]->addOptions( ["UTC"] )
          if $config{clockData}[$i] == 1;
        $typeOms[$i]->addOptions( ["RIG"] )
          if ( $noRig != 1 && $config{rigData}[$i] == 1 );
        $typeOms[$i]->addOptions( ["+1"] )
          if $config{defaultSerialField} eq $config{fieldNames}[$i];

     # Always define textvariable separately, in its own ->configure
     # statement.
        $typeOms[$i]
          ->configure( -textvariable => \$fieldTypes[$i], );

        # Initialise 'type' option  menus by re-assigning values
        # to the textvariables:
        $fieldTypes[$i] = $config{fieldTypes}[$i];

        $typeOms[$i]->configure(
            -command => [
                \&type_change,
                \$fieldTypes[$i],
                \$nqEntry[$i],
                \${ $config{defaultValues} }[$i],
                $mostRecentSerialRef,
            ],
        );

        # Initialise serial number field:
        my $isSerialField = 0;
        if (
            $config{defaultSerialField} eq $config{fieldNames}[$i] )
        {
            if ( $config{debug} ) {
                print "Initialising serial field: i = $i,";
                print "field type = $fieldTypes[$i]\n";
            }
            init_serialfield(
                $nqEntry[$i],    ${ $config{defaultValues} }[$i],
                $fieldTypes[$i], $mostRecentSerialRef,
                $config{debug},
            );
            $isSerialField = 1;
        }

        # Fill field with default values if required.
        #
        if ( $defaultValues[$i] ne "" && $isSerialField == 0 ) {

            # If there is a default value for this entry box,
            # put the value in the entry box,
            $tVars[$i] = $defaultValues[$i]
              if $fieldTypes[$i] eq "---";

            # but don't lock the entry box (yet):
            $typeOms[$i]->configure( -state => 'normal' );
        }

        $nqTable{ $fieldNames[$i] } = $nqEntry[$i];
    }

    $utcCb->configure(
        -command => [
            \&utc_clock,        $hashRef,  \@fieldTypes,
            \$config{utcClock}, \@typeOms, \@nqEntry,
        ],
    );

    # Synchronise 'type' option menus with the 'use UTC clock'
    # checkbutton:
    utc_clock( $hashRef, \@fieldTypes, \$config{utcClock},
        \@typeOms, \@nqEntry, );

    my $dupeColumns = 0;
    foreach $x ( @{ $config{dupeShowFields} } ) {
        $dupeColumns++ if $x == 1;
    }
    my $dupeList = $nqFrameCenter->Scrolled(
        'HList',
        -height           => $config{dupeList_height},
        -scrollbars       => 'osoe',
        -foreground       => $config{dupeList_fg},
        -background       => $config{dupeList_bg},
        -selectforeground => $config{dupeList_sel_fg},
        -selectbackground => $config{dupeList_sel_bg},
        -takefocus        => 0,
        -columns          => $dupeColumns,
        -header           => 1,
        -itemtype         => 'text',
        -selectmode       => 'multiple',
    );

    # Dummy label to push dupe window down:
    $nqFrameCenter->Label(
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    )->pack( -expand => 0, -fill => 'both', -side => 'top' );

    # Put headers on top:
    my $a = 0;
    for ( my $f = 0 ; $f < $#fieldNames ; $f++ ) {
        if ( ${ $config{dupeShowFields} }[$f] == 1 ) {
            $dupeList->headerCreate(
                $a,
                -text             => $fieldDescr[$f],
                -headerbackground => $config{hList_hdr_bg},
            );
            $a++;
        }
    }

    # Add a rowID column at the end of the headers:
    $dupeList->headerCreate(
        $a,
        -text             => "row-ID",
        -headerbackground => $config{rowid_bg},
    );

    my $dupesLabel = $nqFrameCenter->Label(
        -borderwidth => $config{borderwidth},
        -relief      => 'groove',
        -foreground  => $config{fg},
        -background  => $config{bg},
    );

    my $saveButton = $quitFrame->Button(
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -text             => "Return=Save",
        -underline        => '7',
        -command          => [
            \&save_new_qso, \%nqTable, $nqFrame,     \@nqEntry,
            $searchButton,  $window,   \@fieldTypes, $hashRef
        ]
      )->pack(
        -side   => 'left',
        -expand => '1',
        -anchor => 's',
        -fill   => 'x',
      );

    my $deField = 0;    # index number of defaultEntryField.
    for ( my $i = 0 ; $i <= $#fieldNames ; $i++ ) {
        $deField = $i
          if $config{defaultEntryField} eq $fieldNames[$i];
    }

    my $clearButton = $quitFrame->Button(
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -text             => "Wipe",
        -underline        => '0',
        -command          => [
            \&clear_template, $config{defaultValues}, \@nqEntry,
            \@fieldTypes,     $deField,

        ]
      )->pack(
        -side   => 'left',
        -anchor => 's',
        -expand => '1',
        -fill   => 'x',
      );

    # Alt-q|Q generates a $quitButton event:
    $window->bind(
        '<Alt-x>' => [
            sub {
                $quitButton->flash();
                $quitButton->invoke();
              }
        ]
    );
    $window->bind(
        '<Alt-X>' => [
            sub {
                $quitButton->flash();
                $quitButton->invoke();
              }
        ]
    );

    # Set up CW hot-keys at the bottom of the window:
    #
    my $f1Button = $fkeyFrame->Button(
        -text             => 'F1=CQ',
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
      )->pack( -side => 'left', -expand => 1, -fill => 'x' )
      unless $config{noCwDaemon};

    my $autocqRequest = 0;
    my $cqId          = 0;
    my $autocqCb      = $fkeyFrame->Checkbutton(
        -text             => "Auto",
        -variable         => \$autocqRequest,
        -foreground       => $config{fg},
        -background       => $config{bg},
        -activebackground => $config{bg},
        -underline        => '0',
        -command          => [
            sub {

                # Cancel possible scheduled CQ
                # if button has been unchecked.
                # Otherwise: do nothing.
                my ( $autocqRequestRef, $cqIdRef, $debug ) = @_;
                if (   $$autocqRequestRef == 0
                    && $$cqIdRef != 0 )
                {
                    $$cqIdRef->cancel;
                    print "Autocq event \$cqId=$$cqIdRef",
                      " cancelled\n"
                      if $debug;
                }
            },
            \$autocqRequest,
            \$cqId,
            $config{debug}
        ]
      )->pack(
        -side   => 'left',
        -expand => 1,
        -anchor => 'e'
      ) unless $config{noCwDaemon};

    # Make up tag for F2 button. May contain more than one item:
    my $f2Tag = "";
    foreach my $x ( @{ $config{f2FieldDescr} } ) {
        $f2Tag = $f2Tag . $x . "+";
    }
    $f2Tag =~ s/\+$//;

    my $f2Button = $fkeyFrame->Button(
        -text             => "F2=$config{f2Tag}",
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
      )->pack( -side => 'left', -expand => 1, -fill => 'x' )
      unless $config{noCwDaemon};

    my $f3Button = $fkeyFrame->Button(
        -text             => "F3=$config{f3Tag}",
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
      )->pack( -side => 'left', -expand => 1, -fill => 'x' )
      unless $config{noCwDaemon};

    my $f4Button = $fkeyFrame->Button(
        -text             => "F4=$config{f4Tag}",
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
      )->pack( -side => 'left', -expand => 1, -fill => 'x' )
      unless $config{noCwDaemon};

    my $f5Button = $fkeyFrame->Button(
        -text             => "F5=$config{f5Tag}",
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
      )->pack( -side => 'left', -expand => 1, -fill => 'x' )
      unless $config{noCwDaemon};

    my $f6Button = $fkeyFrame->Button(
        -text             => "F6=$config{f6Tag}",
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
      )->pack( -side => 'left', -expand => 1, -fill => 'x' )
      unless $config{noCwDaemon};

    my $f7Button = $fkeyFrame->Button(
        -text             => "F7=$config{f7Tag}",
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
      )->pack( -side => 'left', -expand => 1, -fill => 'x' )
      unless $config{noCwDaemon};

    my $f12Button = $fkeyFrame->Button(
        -text             => "F12=$config{f12Tag}",
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
      )->pack( -side => 'left', -expand => 1, -fill => 'x' )
      unless $config{noCwDaemon};

    my $altkButton = $fkeyFrame->Button(
        -text             => 'Alt-k=CW-kbd',
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
      )->pack( -side => 'left', -expand => 1, -fill => 'x' )
      unless $config{noCwDaemon};

    my $altReturnButton = $fkeyFrame->Button(
        -text             => 'Alt-Return=Save+TU',
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
      )->pack( -side => 'left', -expand => 1, -fill => 'x' )
      unless $config{noCwDaemon};

    my @cwButtons = (
        $abbrevCb,  $wpmOptionMenu, $f1Button,
        $f2Button,  $f3Button,      $f4Button,
        $f5Button,  $f6Button,      $f7Button,
        $f12Button, $altkButton,    $altReturnButton,
    ) unless $config{noCwDaemon};

    # Set up key bindings for logger panel:
    #
    do {
        $window->bind(
            '<F1>' => sub {
                $f1Button->flash;
                $f1Button->invoke;
            }
        );

        $window->bind(
            '<F2>' => sub {
                $f2Button->flash;
                $f2Button->invoke;
            }
        );

        $window->bind(
            '<F3>' => sub {
                $f3Button->flash;
                $f3Button->invoke;
            }
        );

        $window->bind(
            '<F4>' => sub {
                $f4Button->flash;
                $f4Button->invoke;
            }
        );

        $window->bind(
            '<F5>' => sub {
                $f5Button->flash;
                $f5Button->invoke;
            }
        );

        $window->bind(
            '<F6>' => sub {
                $f6Button->flash;
                $f6Button->invoke;
            }
        );

        $window->bind(
            '<F7>' => sub {
                $f7Button->flash;
                $f7Button->invoke;
            }
        );

        $window->bind(

            # F12 generates F5 and F2 events:
            '<F12>' => sub {
                $f12Button->flash;
                $f12Button->invoke();
            }
        );

        $altkButton->configure(
            -command => sub {
                if ( !Exists($cwWindow) ) {
                    \&cw_kbd(
                        $window,    \%config,
                        \$cwWindow, $cwdSocket
                    );
                }
                else {
                    $cwWindow->raise;
                    $cwWindow->focus;
                }
            }
        );

        $altReturnButton->configure(
            -command => sub {
                $saveButton->invoke();
                $f3Button->flash;
                $f3Button->invoke();

            }
        );

        $window->bind(

            # Log QSO and send TU msge:
            '<Alt-Return>' => sub {
                $altReturnButton->flash();
                $altReturnButton->invoke();
            }
        );

    } unless $config{noCwDaemon};

    # Log QSO and send TU msge:
    $window->bind(
        '<Return>' => sub {
            my $dontLogIt = 0;

            # Check if we're wanting to set QRG or mode:
            # Scan array @fieldNames to find callsign field:
            for ( my $i = 0 ; $i <= $#fieldNames ; $i++ ) {
                if ( $fieldNames[$i] =~ /($config{isCall})/ ) {

                    # This field is the callsign field:
                    my $callSign = $nqEntry[$i]->get;
                    $dontLogIt = set_rig_data(
                        $rigctldSocket, $noRig, $config{debug},
                        $callSign,      $clearButton,
                    );
                }
            }
            do {

                # Save QSO unless we're trying to set QRG or mode:
                $saveButton->flash();
                $saveButton->invoke();
            } unless $dontLogIt;
        }
    );

    # Alt-s|S generates a $saveButton event:
    $window->bind(
        '<Alt-s>' => sub {
            $saveButton->flash;
            $saveButton->invoke();
        }
    );

    $window->bind(
        '<Alt-S>' => sub {
            $saveButton->flash;
            $saveButton->invoke();
        }
    );

    $window->bind(
        '<Alt-u>' => sub {
            $utcCb->invoke();
        }
    );

    $window->bind(
        '<Alt-U>' => sub {
            $utcCb->invoke();
        }
    );

    $window->bind(
        '<Alt-w>' => sub {
            $clearButton->flash;
            $clearButton->invoke();
        }
    );

    $window->bind(
        '<Alt-W>' => sub {
            $clearButton->flash;
            $clearButton->invoke();
        }
    );

    $window->bind(
        '<Alt-A>' => sub {
            if ( $autocqRequest == 1 ) {
                $autocqRequest = 0;
            }
            else {
                $autocqRequest = 1;
            }
        }
    );

    $window->bind(
        '<Alt-a>' => sub {
            if ( $autocqRequest == 1 ) {
                $autocqRequest = 0;
            }
            else {
                $autocqRequest = 1;
            }
        }
    );

    $window->bind(
        '<Alt-B>' => sub {
            $abbrevCb->invoke();
        }
    );

    $window->bind(
        '<Alt-b>' => sub {
            $abbrevCb->invoke();
        }
    );

    $window->bind(

   # Jump to 'defaultEntryField' (typically the callsign entry box):
        '<Alt-C>' => [
            sub {
                $nqEntry[$deField]->focus();
              }
        ],
    );

    $window->bind(

   # Jump to 'defaultEntryField' (typically the callsign entry box):
        '<Alt-c>' => [
            sub {
                $nqEntry[$deField]->focus();
              }
        ],
    );

    $window->bind(
        '<Alt-L>' => sub {
            $leadingZerosCb->invoke();
        }
    );

    $window->bind(
        '<Alt-l>' => sub {
            $leadingZerosCb->invoke();
        }
    );

    require Tk::Widget;
    $window->bind(
        '<Escape>' => sub {

            # Aborts CW stream by interrupting keyer:
            my $cwdEsc = "\033";
            print "Aborting CW\n" if $config{debug};
            my $msge = $cwdEsc . $config{cwdAbortPrefix};
            send_msge(
                undef,      "$msge", \@cwButtons, $hashRef,
                $cwdSocket, undef,   undef,       undef,
                undef,
            );

            # Uncheck the autoCQ button:
            $autocqRequest = 0;
            if ( $cqId != 0 ) {
                print "\$cqId=$cqId cancelled.\n" if $config{debug};
                $cqId->cancel;
            }
          }    # End of sub <Escape>
    ) unless $config{noCwDaemon};

    if ( $config{debug} ) {
        print "utcClock = $config{utcClock}\n";
        print "useRig   = $config{useRig}\n";
    }

    if ( $noRig == 1 ) {
        $config{useRig} = 0 if $noRig == 1;
        print
"Rig not wanted, not found, or no hamlib: useRig turned off\n"
          if $config{debug};
    }

    if ( $config{noCwDaemon} != 1 ) {
        $f1Button->configure(
            -command => [
                \&send_msge,     undef,      $config{CQ},
                \@cwButtons,     $hashRef,   $cwdSocket,
                \$autocqRequest, \$f1Button, \$fkeyFrame,
                \$cqId,
            ],
        );

        $f2Button->configure(
            -command => sub {
                send_field(
                    undef,
                    \$nqEntry[ $config{f2FieldNrA} ],
                    \@cwButtons,
                    $hashRef,
                    $config{abbrevNum},
                    $config{leadingZeros},
                    $config{f2FieldNrA},
                    $cwdSocket
                ) unless $autoCQ;
                sleep
                  1;   # This stops cwdaemon from losing the plot...

                send_field(
                    undef,
                    \$nqEntry[ $config{f2FieldNrB} ],
                    \@cwButtons,
                    $hashRef,
                    $config{abbrevNum},
                    $config{leadingZeros},
                    $config{f2FieldNrB},
                    $cwdSocket
                );
            },
        );

        $f3Button->configure(
            -command => [
                \&send_msge,    undef,
                $config{f3Tag}, \@cwButtons,
                $hashRef,       $cwdSocket,
                undef,          undef,
                undef,          undef,
            ],
        );

        $f4Button->configure(
            -command => [
                \&send_msge,    undef,
                $config{f4Tag}, \@cwButtons,
                $hashRef,       $cwdSocket,
                undef,          undef,
                undef,          undef,
            ],
        );

        $f5Button->configure(
            -command => [
                \&send_field,
                undef,
                \$nqEntry[ $config{f5FieldNr} ],
                \@cwButtons,
                $hashRef,
                0,
                \$config{lzFields},
                $config{f5FieldNr},
                $cwdSocket
            ]
        );

        $f6Button->configure(
            -command => [
                \&send_msge,    undef,
                $config{f6Tag}, \@cwButtons,
                $hashRef,       $cwdSocket,
                undef,          undef,
                undef,          undef,
            ],
        );

        $f7Button->configure(
            -command => [
                \&send_msge,    undef,
                $config{f7Tag}, \@cwButtons,
                $hashRef,       $cwdSocket,
                undef,          undef,
                undef,          undef,
            ],
        );

        $f12Button->configure(
            -command => [
                sub {
                    print "executing F12\n" if $config{debug};
                    send_field(
                        undef,
                        \$nqEntry[ $config{f12FieldNrA} ],
                        \@cwButtons,
                        $hashRef,
                        0,
                        0,
                        $config{f12FieldNrA},
                        $cwdSocket
                    );
                    sleep 1
                      ; # This stops cwdaemon from losing the plot...
                    send_field(
                        undef,
                        \$nqEntry[ $config{f12FieldNrB} ],
                        \@cwButtons,
                        $hashRef,
                        $config{abbrevNum},
                        $config{leadingZeros},
                        $config{f12FieldNrB},
                        $cwdSocket
                    );
                    sleep 1
                      ; # This stops cwdaemon from losing the plot...
                    send_field(
                        undef,
                        \$nqEntry[ $config{f12FieldNrC} ],
                        \@cwButtons,
                        $hashRef,
                        $config{abbrevNum},
                        $config{leadingZeros},
                        $config{f12FieldNrC},
                        $cwdSocket
                    );
                  }
            ]
        );

        send_msge(
            undef,      "qrv", \@cwButtons, $hashRef,
            $cwdSocket, undef, undef,       undef,
            undef,
        ) if $config{qrvSk} == 1;

        # Create CW keyboard:
        my $cwWindow = undef;
        $window->bind(
            '<Alt-k>' => sub {
                $altkButton->flash;
                $altkButton->invoke;
            }
        );
        $window->bind(
            '<Alt-K>' => sub {
                $altkButton->flash;
                $altkButton->invoke;
            }
        );

    }

    # Pack everything:
    $nqFrame->pack();
    $dupesLabel->pack(
        -side   => 'bottom',
        -anchor => 'n',
        -expand => '0',
        -fill   => 'x',
    );

    $dupeList->Subwidget("yscrollbar")->configure(
        -activebackground => $config{sb_activebackground},
        -background       => $config{sb_background},
        -takefocus => 0,    # Don't tab into scrollbar

    );
    $dupeList->Subwidget("xscrollbar")->configure(
        -activebackground => $config{sb_activebackground},
        -background       => $config{sb_background},
        -takefocus => 0,    # Don't tab into scrollbar
    );

    $dupeList->pack(
        -expand => 1,
        -fill   => 'both',
        -anchor => 's',
        -side   => 'bottom',
    );

    # Refresh timestamps and rig data regularly, and update the dupe
    # sheet:
    my $oldCall =
      "NOCAL";    # Used to detect whether the value in the call
                  # entry box has changed. If so, this will trigger
                  # a dupe sheet refresh.
    my $tsId = $window->repeat(
        $config{tsRefresh},
        [
            \&update_tsentries, \@nqEntry, $hashRef,
            \@fieldTypes,       \$dxccStr
        ],
    );

    # Refresh DXCC info:
    my $dxccId = $window->repeat(
        $config{dxccRefresh},
        [
            \&update_dxcc, \@nqEntry, $hashRef,
            \@fieldNames,  \$dxccStr, \%ctyHash,
            \%contHash,    \%cqzHash, \%ituzHash
        ],
    );

    # Refresh Super Check Partial info:
    my $scpId = $scpWindow->repeat(
        $config{scpRefresh},
        [
            \&update_scp, \@nqEntry,  $hashRef,
            \@fieldNames, \@scpArray, \$scpString
        ],
    ) if $doScp == 1;

    $scpWindow->OnDestroy(
        sub {
            $doScp = 0;
            $scpId->time(undef);
            print "SCP destroyed, cancelling SPC event.\n"
              if $config{debug};
        }
    );

    my $rigId;
    my $rigErrCount =
      0;    # Records _successive_ failures of CAT queries
    $rigId = $window->repeat(
        $config{rigRefresh},
        [
            \&update_rigentries, \@nqEntry,
            \$rigctldSocket,     $hashRef,
            \$noRig,             \@cwButtons,
            \@fieldTypes,        $window,
            \@typeOms,           \$rigLock,
            \$rigId,             \$rigErrCount,
        ],
    );

    my $dupeId = $window->repeat(
        $config{dupeRefresh},
        [
            \&update_dupesheet, \@nqEntry,   $hashRef,
            $dupeList,          $dupesLabel, \$oldCall,
            \@fieldTypes,       \$window,
        ],
    ) or die;

    # Kill all '->repeat' events if $window is zapped by the user.
    # $id->cancel does not appear to work at all, so I've tried
    # id->time(undef), which does work...
    $window->OnDestroy(
        sub {
            print "Cancelling \'repeat\' events.\n"
              if $config{debug};
            $dxccId->time(undef);
            $rigId->time(undef);
            $tsId->time(undef);
            $dupeId->time(undef);
        }
    );

    # Simulate a field type 'change' on all entry boxes to make sure
    # that the state of the entry boxes (normal for CAR or disabled
    # when LCK, UTC etc) is correct:
    for ( my $i = 0 ; $i <= $#nqEntry ; $i++ ) {
        type_change( \$fieldTypes[$i], \$nqEntry[$i],
            \${ $config{defaultValues} }[$i],
            $mostRecentSerialRef );
    }

    # Force timestamp update _now_ so we won't have to wait
    # for a tsRefresh event to see correct UTC and date...
    update_tsentries( \@nqEntry, $hashRef, \@fieldTypes,
        \$dxccStr );

    # Force QRG en mode update _now_ so we won't have to wait
    # for a rigRefresh event to see correct rig data...
    update_rigentries(
        \@nqEntry, \$rigctldSocket, $hashRef,
        \$noRig,   \@cwButtons,     \@fieldTypes,
        $window,   \@typeOms,       \$rigLock,
        \$rigId,   \$rigErrCount,
    );

   # Return the nqEntry array, used to put the focus on defaultEntry
   # once the copyright window is out of the way.

    # print "ser: $$mostRecentSerialRef\n";

    return ( \@nqEntry );
}

######################################################################
#
# subroutine send_msge
#
# Purpose: Transmits CW message on cwdaemon UDP port. If transmission
#          to cwdaemon somehow fails, then all CW function keys are
#          disabled.
#
# Args:    1. First argument, not used.
#          2. Text string which will be transmitted in CW.
#          3. Reference to an array containing instances of all CW
#             function buttons. May be undef.
#          4. Reference to hash  containing a copy of config file
#             variables.
#          5. Internet socket on which to transmit to cwdaemon.
#          6. Reference to variable that holds the state of the
#             auto-CQ button. If '1', then the op wants to
#             automatically call CQ. May be undef.
#          7. Reference to the CQ button(F1). The button is 'invoked'
#             by the auto-CQ process. May be undef.
#          8. Reference to scalar $cqId. This scalar gets to hold the
#             ID of
#             the autoCQ process, so that it can be cancelled with
#             the ESC key.
#
#
# Returns: void
#
######################################################################
#
sub send_msge {

    my (
        $junk,        $msge,         $cwButtonsRef,
        $hashRef,     $cwdSocket,    $autocqRequestRef,
        $f1ButtonRef, $fkeyFrameRef, $cqIdRef
    ) = @_;

    my %config = %{$hashRef};
    my $debug  = $config{debug};
    return if $config{noCwDaemon};

    # Cancel auto-CQ: we're about to transmit CW and don't
    # want a collision with scheduled CQs...
    if ( defined $$cqIdRef && $$cqIdRef != 0 ) {
        $$cqIdRef->cancel;
        print "\$cqId=$$cqIdRef cancelled.\n" if $config{debug};
    }

    my $s = "$config{cwdHost}:$config{cwdPort}";
    if ( !( print $cwdSocket $msge ) ) {
        if ( $config{debug} ) {
            print
"Sending \'$msge\' on cwdaemon UDP socket $s failed.\n";
            print "Turned off cwdaemon support.\n";
        }
        ${$hashRef}{noCwDaemon} = 1;
        if ( defined($cwButtonsRef) ) {
            foreach my $button ( @{$cwButtonsRef} ) {
                $button->configure( -state => "disabled" );
            }
        }
    }
    else {
        print "Sent \'$msge\' on cwdaemon UDP socket $s.\n"
          if $config{debug};
    }

    # If the automatic CQ button $autocqRequest is active
    # then schedule another delayed (auto) CQ.
    if (   defined($autocqRequestRef)
        && defined($$fkeyFrameRef)
        && $$autocqRequestRef == 1 )
    {
        $$cqIdRef = $$fkeyFrameRef->after(
            $config{cqRepeat} * 1000,
            [
                sub {
                    my ($f1ButtonRef) = @_;

                    # Once the event has been scheduled,
                    # it cannot be unscheduled, so we only invoke
                    # f1Button if the autocq button is checked at
                    # the time. If it isn't, then the event still
                    # runs at the appointed time, but without CQ!
                    if ( $$autocqRequestRef == 1 ) {
                        $$f1ButtonRef->invoke;
                        $$f1ButtonRef->flash;
                    }
                },
                $f1ButtonRef,
                $autocqRequestRef
            ]
        );
        print "Delayed CQ \$cqId=$$cqIdRef scheduled in ",
          $config{cqRepeat}, " seconds from now...\n"
          if $debug == 1;
    }
}

######################################################################
#
# subroutine send_field
#
# Purpose: Transmits contents of new QSO Entry field on cwdaemon UDP
#          port. If the Entry field requires it, 'leading zero' and
#          'abbreviated number' action is applied before sending the
#          string to cwdaemon. Should transmission to  cwdaemon
#          somehow fail, then the CW function buttons are disabled.
#
# Args:    1. First argument, unused (callback cruft).
#          2. Reference to Entry box, which holds a string that is to
#             be transmitted in CW.
#          3. Reference to an array containing instances of all CW
#             function buttons.
#          4. Reference to hash that contains configuration scalars.
#          5. Integer that indicates (if '1') that numbers are to be
#             sent in abbreviated fashion (e.g '0' as 'T').
#          6. Integer that indicates (if '1') that leading zeros in a
#             serial number are to be sent. (5) applies to these
#             leading zeros.
#          7. Integer that is the index number of the field that
#             holds the the serial number. This is the value of
#             'f2FieldA' in the config file
#          8. Internet socket on which to transmit to cwdaemon
#
# Returns: void
#
######################################################################
#
sub send_field {
    my ( $junk, $fieldRef, $cwButtonsRef, $hashRef, $abbrevNum,
        $leadingZeros, $i, $cwdSocket )
      = @_;

    my %config = %{$hashRef};
    my $debug  = $config{debug};

    return if $config{noCwDaemon};
    my $msge = ${$fieldRef}->get;

    foreach my $l ( @{ $config{lzFields} } ) {
        if ( $leadingZeros != '1' ) {

            # Kill leading zeros - typically in serial number sent:
            $msge =~ s/^0+// if $l == 1;
        }
    }

    foreach my $a ( @{ $config{abbrevFields} } ) {
        if ( $abbrevNum == 1 && $i == $a ) {

         # The user wants numbers to be sent in abbreviated fashion:
            $msge =~ s/0/T/g;
            $msge =~ s/9/N/g;
            $msge =~ s/1/A/g;
        }
    }

    my $s = "$config{cwdHost}:$config{cwdPort}";
    if ( !( print $cwdSocket $msge ) ) {

        if ( $config{debug} ) {
            print
"Sending \'$msge\' on cwdaemon UDP socket $s failed.\n";
            print "Turning off cwdaemon support.\n";
        }
        ${$hashRef}{noCwDaemon} = 1;

        if ( defined($cwButtonsRef) ) {
            foreach my $button ( @{$cwButtonsRef} ) {
                $button->configure( -state => "disabled" );
            }
        }

    }
    else {
        print "Sent \'$msge\' on cwdaemon UDP socket $s.\n"
          if $config{debug};
    }
}

######################################################################
#
# callback type_change
#
# Purpose: Executes when the user accesses a 'type' option menu.
#
#          If 'LCK', the entry box is disabled for user access: the
#          value is locked-in.
#
#          If the option menu has been changed to 'CAR', then user
#          access to the entry box is restored.
#
#
# Args:
#
#     1. Reference to the value of a 'type' option menu. This is
#     the menu value for the field. The subroutine uses this reference
#     to save the entry box data and enable or disable acces to it.
#
#     2. Instance of an entry box. This is an entry box in the 'new
#     QSO' template.
#
#     3. The default value for the entry box. Defined in the config
#     file.  This may be a default frequency, day, month, year, UTC or
#     mode.
#
#
######################################################################
#
sub type_change {

    my (
        $fieldTypeRef,    $nqEntryRef,
        $defaultValueRef, $mostRecentSerialRef
    ) = @_;

    if ( $$fieldTypeRef eq "LCK" ) {

       # The user wants to lock the present value into the next QSO.
       # Disable access to the entry field.
        $$nqEntryRef->configure( -state => 'disabled' );
        return;
    }

    if ( $$fieldTypeRef eq "RIG" ) {

       # The user wants to lock the present value into the next QSO.
       # Disable access to the entry field.
        $$nqEntryRef->configure( -state => 'disabled' );
        return;
    }

    if ( $$fieldTypeRef eq "CAR" ) {

 # The user wants to carry over the present value into the next QSO:
 # Enable access to the entry field.
        $$nqEntryRef->configure( -state => 'normal' );
        return;
    }

    if ( $$fieldTypeRef eq "+1" ) {

        # We're dealing with an incrementing number field:
        init_serialfield( $$nqEntryRef, $$defaultValueRef, "+1",
            $mostRecentSerialRef, $config{debug} );
        return;
    }

    if ( $$fieldTypeRef eq "UTC" ) {

        # The PC clock is automatically writes time && date, so
        # disable access to the entry field.
        $$nqEntryRef->configure( -state => 'disabled' );
        return;
    }

    # In all other cases of field type:
    # Make entry box accessible, and leave the contents as-is:
    $$nqEntryRef->configure( -state => 'normal' );

    return;
}

######################################################################
#
# subroutine check_entry
#
# Purpose: Checks validity of character entered into the 'new QSO'
#          entry boxes. Filters and alters (e.g. uppercase)
#          on-the-fly. This subroutine is typically called by the
#          'validatecommand' of an Entry.
#
# Args:
#
#
#   1. Reference to hash that contains configuration scalars.
#   2. An integer. This is the index number of the logfield under
#      investigation.
#   3. Instance of an entry box. This is the entry box that we are
#      typing into.
#   4. Reference to the variable that holds the entry box contents.
#      The subroutine uses it to display the filtered, new string.
#   5. The newly entered string.
#   6. The most recently typed character.
#   7. The old string, before the user typed something.
#   8. Index of the cursor position.
#   9. If we are inserting something into a string, then this value
#      is '1'.
#      If we are deleting, then this value is not '0'.
#
# Returns: true ('1') in case of a delete action, which is handled by
#          the Entry box. False ('0') otherwise, because anything
#          other than a delete operation is fully taken care of by
#          the subroutine.
#
######################################################################
#
sub check_entry {

    my (
        $autoCQRef, $hashRef,   $i,            $e,
        $tVarRef,   $newString, $newCharacter, $oldString,
        $charIndex, $insertMode,
    ) = @_;

    # print "Testing insert or delete...\n";

    # If we are deleting something, then just do it:
    return (1) if ( $insertMode != 1 && $insertMode != 8 );

    # PS: != 8 is a workaround for a Perl/Tk bug...

    # We're inserting a character:

    my %config = %{$hashRef};

    my @fieldDescr = @{ $config{fieldDescr} };
    my $fd         = $fieldDescr[$i];

    # Don't allow values to start with whitespaces.
    # This can cause trouble when searching the log.
    if ( $charIndex == 0 && $newCharacter =~ /^\s/ ) {
        return (0);
    }

    # Don't allow the escape character...
    if ( $newCharacter =~ /\\/ ) {
        return (0);
    }

    my @spaceOK = @{ $config{spaceOK} };
    if ( $spaceOK[$i] == 0 ) {
        if ( $newCharacter eq " " ) {

            # No space allowed in this field.
            # Don't allow char to be entered. Ignore.
            return (0);
        }
    }

    my @intAll = @{ $config{intAll} };
    if ( $intAll[$i] == 1 ) {

        # Entry must be numeric:
        if ( $newCharacter !~ /[0-9]/ ) {

            # Don't allow char to be entered. Ignore.
            return (0);
        }
    }

    my @capStart = @{ $config{capStart} };
    if ( $capStart[$i] == 1 ) {
        $newString = ucfirst($newString);
    }

    my @capAll = @{ $config{capAll} };
    if ( $capAll[$i] == 1 ) {

        # Entry must be all capitals:
        $newString = uc($newString);
    }

    # Accept the new string:
    $$tVarRef = $newString;

    # Advance insertion cursur.
    # We will have to do this ourselves because we are
    # working through the text variable $tVar instead of $e->insert
    # :-(
    $e->icursor( $charIndex + 1 );

    # For some strange reason, Perl/Tk 804.027 needs this:
    $e->configure( -validate => 'key' );

    # Turn off autoCQ if we're typing in the callsign field:
    my @fieldNames = @{ $config{fieldNames} };
    if ( $fieldNames[$i] =~ /($config{isCall})/ ) {
        $$autoCQRef = 0 if defined $autoCQRef;
    }

    return (0);
}

######################################################################
#
# subroutine update_tsentries
#
# Purpose: If required, updates time-related or DXCC-related entry
#          boxes with current time-stamp or DXCC info:
#
# Args:
#
#   1. Reference to array of 'new QSO' entry box widgets. These are
#      the entry fields of the new QSO template.
#   2. Reference of hash containing the config values.
#   3. Reference to array containing the 'type' for each field, as per
#      config file. However, the type of a field can be changed by the
#      user during program execution and we are using the actual
#      value.
#   4. Reference to scalar $dxccStr which holds the DXCC info string.
#
# Returns: void.
#
######################################################################
#
sub update_tsentries {

    my ( $nqEntryRef, $hashRef, $fieldTypesRef, $dxccStrRef ) = @_;

    my %config = %{$hashRef};

    my @nqEntry    = @{$nqEntryRef};
    my @fieldNames = @{ $config{fieldNames} };

    # Now update date/time/mode/khz into appropriate entry
    # boxes like this:
    # Enable entry box, wipe contents, put in updated value
    # and disable box.
    for ( my $i = 0 ; $i <= $#fieldNames ; $i++ ) {
        if ( ${$fieldTypesRef}[$i] =~ /(UTC)/ ) {

            # The field is a clock field or a DXCC field:
            my ( $dd, $mm, $yyyy, $utc ) = grab_utc();
            refresh_field( $nqEntry[$i], $dd )
              if $fieldNames[$i] eq $config{isDay};
            refresh_field( $nqEntry[$i], $mm )
              if $fieldNames[$i] eq $config{isMonth};
            refresh_field( $nqEntry[$i], $yyyy )
              if $fieldNames[$i] eq $config{isYear};
            refresh_field( $nqEntry[$i], $utc )
              if $fieldNames[$i] eq $config{isUtc};
            refresh_field( $nqEntry[$i], $$dxccStrRef )
              if $fieldNames[$i] eq $config{isCountry};
            refresh_field( $nqEntry[$i], $$dxccStrRef )
              if $fieldNames[$i] eq $config{isContinent};

        }
    }
}

######################################################################
#
# subroutine update_dxcc
#
# Purpose: Updates dxcc info in status bar, provided the callsign
#          field holds a call.
#
# Args:
#
#   1. Reference to array of 'new QSO' entry box widgets. These are
#      the entry fields of the new QSO template.
#   2. Reference of hash containing the config values.
#   3. Reference to array containing the 'type' for each field, as per
#      config file. However, the type of a field can be changed by the
#      user during program execution and we are using the actual
#      value.
#   4. Reference to scalar which holds (and prints) the DXCC info
#      string.
#
# Returns: void.
#
######################################################################
#
sub update_dxcc {

    my (
        $nqEntryRef, $hashRef,    $fieldNamesRef,
        $dxccStrRef, $ctyHashRef, $contHashRef,
        $cqzHashRef, $ituzHashRef
    ) = @_;

    my %config     = %{$hashRef};
    my @nqEntry    = @{$nqEntryRef};
    my @fieldNames = @{$fieldNamesRef};

    # Scan array @fieldNames to find callsign field:
    for ( my $i = 0 ; $i <= $#fieldNames ; $i++ ) {

        if ( $fieldNames[$i] =~ /($config{isCall})/ ) {

            # This field is the callsign field:
            do {
                if ( $config{debug} ) {
                    print "Could not retrieve entry from entry box";
                    print " in update_dxcc\(\).\n";
                }
                return;
              }
              unless eval { defined $nqEntry[$i]->get };
            my $callSign;
            $callSign = $nqEntry[$i]->get;

            #	  print "get returns: $callSign\n";
            #	      or die "Error in update_dxcc";
            my $cty   = "";
            my $cont  = "";
            my $cqz   = "";
            my $ituz = "";
            my $ut    = "";

            # Request DXCC info on callsign (provided not empty):
            $$dxccStrRef = "DXCC info bar";

            if ( $callSign ne "" ) {

                # Check to see if callsign produces a direct match:
                if ( defined $$ctyHashRef{$callSign} ) {
                    $cty  = $$ctyHashRef{$callSign};
                    $cont = $$contHashRef{$callSign}
                      if defined $$contHashRef{$callSign};
                    $cqz = $$cqzHashRef{$callSign}
                      if defined $$cqzHashRef{$callSign};
                    $ituz = $$ituzHashRef{$callSign}
                      if defined $$ituzHashRef{$callSign};
                    $$dxccStrRef =
                      "CTY: $cty - CONT: $cont - CQZ: $cqz ";
                    $$dxccStrRef = $$dxccStrRef . "- ITUZ: $ituz";
                }
                else {

                    # Get rid of confusing stuff like:
                    # /P /M /MM /QRP /A /<single-digit>:
                    $callSign =~
                      s/\/P$|\/M$|\/MM$|\/QRP$|\/A$|\/\d$//;
                    my $x = $callSign;

                    if ( $callSign =~ m/^(.+)\/(.+)$/ ) {

                       # print "\$1=$1, \$2=$2\n" if $config{debug};
                       # The callsign is of the form $1/$2
                        if ( length $1 < length $2 ) {

                       # $1 is shorter, so it's probably the country
                            $ceptCountry = $1;
                            $callSign    = $2;
                            $x           = $ceptCountry;
                        }
                        else {

                    # $1 and $2 are equal in length or $1 is longest
                    # so probably $2 is the cty
                            $callSign    = $1;
                            $ceptCountry = $2;
                            $x           = $ceptCountry;
                        }
                    }

                    my $found = 0;
                    my $l     = length($x);
                    do {
                        $x = substr( $x, 0, $l );
                        $found = 1 if defined $$ctyHashRef{$x};
                        $l--;
                    } while ( $found == 0 && $l > 0 );

                    $cty = $$ctyHashRef{$x}
                      if defined $$ctyHashRef{$x};
                    $cont = $$contHashRef{$x}
                      if defined $$contHashRef{$x};
                    $cqz = $$cqzHashRef{$x}
                      if defined $$cqzHashRef{$x};
                    $ituz = $$ituzHashRef{$x}
                      if defined $$ituzHashRef{$x};

                    my $tmp = "CTY: $cty - CONT: $cont - CQZ: ";
                    $$dxccStrRef = $tmp . "$cqz - ITUZ: $ituz";
                }
            }

         # Fill in or wipe contents of continent and country fields:
            for ( my $zz = 0 ; $zz <= $#fieldNames ; $zz++ ) {
                if ( $fieldNames[$zz] =~ /$config{isCountry}/ ) {
                    refresh_field( $$nqEntryRef[$zz], "$cty" );
                }
            }

            for ( my $zz = 0 ; $zz <= $#fieldNames ; $zz++ ) {
                if ( $fieldNames[$zz] =~ /$config{isContinent}/ ) {
                    refresh_field( $$nqEntryRef[$zz], "$cont" );
                }
            }

        }
    }
}

# End of sub update_dxcc

######################################################################
#
# subroutine update_rigentries
#
# Purpose: Updates kHz and mode from rig (if rig is connected).
#
# Args:
#
#   1. Reference to array of 'new QSO' entry box widgets. These are
#      the entry fields of the 'new QSO' template.
#   2. Refernce to instance of open socket to rigctld
#   3. Reference of hash containing the config values.
#   4. Reference to a variable which - if '1' - indicates there is no
#      rig
#      connected, or the user doesn't want to use a rig.
#   5. Reference to array containing instances of buttons that apply
#      only to CW operation.
#   6. Reference to array containing the 'type' for each field, as per
#      config file. However, the type of a field can be changed by the
#      user during program execution and we are using the actual
#      value.
#   7. Instance of a window (to be used to pop up warnings)
#   8. Reference to array. Push any instances of warning/error
#      windows on to this array so that they can be put to the top of
#      the window stack.
#   9. Reference to scalar which, if '1', causes the rig poll to be
#      aborted.
#  10. Reference to scalar which is an instance of the rig poll
#      'repeat' event (used to cancel timed rig polling)
#  11. Reference to scalar which records how often in succession a
#      rig query has failed.
#
# Returns: void.
#
######################################################################
#
sub update_rigentries {

    my (
        $nqEntryRef, $rigctldSocketRef, $hashRef,
        $noRigRef,   $cwButtonsRef,     $fieldTypesRef,
        $window,     $typeOmsRef,       $rigLockRef,
        $rigIdRef,   $rigErrCountRef
    ) = @_;

    my %config        = %{$hashRef};
    my $rigctldSocket = $$rigctldSocketRef;

    my $debug      = $config{debug};
    my @nqEntry    = @{$nqEntryRef};
    my @fieldNames = @{ $config{fieldNames} };
    my @fieldDescr = @{ $config{fieldDescr} };

    print "Executing update_rigentries\(\)\n" if $debug;
    if ( $$rigLockRef == 1 ) {
        if ($debug) {
            print
              "Rig is momentarily locked in update_rigentries. ";
            print "Skipping poll.\n";
        }
        return;
    }
    $$rigLockRef = 1;
    print "update_rig_entries(): polling rig."
      . " \$rigErrCount=$$rigErrCountRef\n"
      if $debug;

    # Now update mode/khz in appropriate entry boxes like this:
    # Enable entry box, wipe contents, put in updated value, disable box.
    my $khz;
    my $mode;
    my $err;
    if ( $$noRigRef != 1 ) {

        my ( $khz, $mode, $err ) =
          get_rig_data( $rigctldSocket, $debug );

        if   ( $err ne "" ) { $$rigErrCountRef++; }
        else                { $$rigErrCountRef = 0; }

        if ( $$rigErrCountRef > $config{maxRigErrCount} ) {
            $$noRigRef = 1;        # turn off rig support....
            $$rigIdRef->cancel;    # Kill rig poll event
            $errMsge =
              "Warning:\n get_rig_data() returned:\n $err \n\n";
            $errMsge =
              $errMsge . "Rig polling is now turned OFF.\n\n";
            $errMsge = $errMsge . "Removing field type \'RIG\'.\n";
            warn "$errMsge";
            my_dialog( $hashRef, $window, "Warning", "Close",
                $config{dialogGeometry}, "$errMsge" );

            for ( my $i = 0 ; $i <= $#fieldNames ; $i++ ) {
                if ( ${$fieldTypesRef}[$i] eq "RIG" ) {

                    ${$fieldTypesRef}[$i] = "CAR";

                   # TODO: remove option 'RIG' from the pick list.
                   # Alas, TK::Optionmenu has no method for removing
                   # options :-( ..so we defining a new pick list
                   # without 'RIG':
                    ${$typeOmsRef}[$i]->configure(
                        -options => [ "CAR", "LCK", "---" ], );
                    ${$nqEntryRef}[$i]
                      ->configure( -state => 'normal' );
                }
            }
        }
        else {
            for ( my $i = 0 ; $i <= $#fieldNames ; $i++ ) {
                if ( ${$fieldTypesRef}[$i] eq "RIG" ) {

                    # The field contains rig data:
                    refresh_field( $nqEntry[$i], $khz )
                      if $fieldNames[$i] eq $config{isKhz};
                    refresh_field( $nqEntry[$i], $mode )
                      if $fieldNames[$i] eq $config{isMode};
                }
            }
        }
    }

    # Update state of CW function keys:
    #
    if ( $config{noCwDaemon} == 1 ) {

        # Disable CW buttons because there is no cwdaemon:
        foreach my $k ( @{$cwButtonsRef} ) {
            $k->configure( -state => 'disabled' );
        }
    }
    else {

        # We have cwdaemon; now find the mode field:
        for ( my $i = 0 ; $i <= $#nqEntry ; $i++ ) {
            if ( $fieldNames[$i] eq $config{isMode} ) {

                # ..and check what mode the rig is in:
                eval { defined $nqEntry[$i]->get } or return;
                if ( $nqEntry[$i]->get =~ /cw/i ) {

                    # Yep...we're on CW. Activate CW function keys:
                    foreach my $k ( @{$cwButtonsRef} ) {
                        $k->configure( -state => 'normal' );
                    }
                }
                else {

                  # Disable CW buttons because the rig is not on CW:
                    foreach my $k ( @{$cwButtonsRef} ) {
                        $k->configure( -state => 'disabled' );
                    }
                }
            }
        }
    }

    $$rigLockRef = 0;
}    # end of sub update_rigentries

######################################################################
#
# subroutine update_dupesheet
#
# Purpose: Re-writes the dupe sheet if the callsign entered has
#          changed.
#          Wipes the dupe sheet if callsign entry box is empty.
# Args:
#
#   1. Reference to array of 'new QSO' entry box widgets. These are
#      the entry fields of the new QSO template.
#   2. Reference of hash containing the config values.
#   3. Instance of a 'List' widget containing QSOs made before with
#      the callsign just entered in the callsign field. This is the
#      list in the dupe window.
#   4. String of text we want to appear under the dupe window.
#      Typically, this is something like '- dupe window -'.
#   5. Reference to call found during previous dupe-check (by this
#      subroutine).
#   6. Reference to array containing the 'type' for each field, as per
#      config file. However, the type of a field can be changed by the
#      user during program execution and we are using the actual
#      value.
#   7. Reference to instance of a window.
#
# Returns: void.
#
######################################################################
#
sub update_dupesheet {

    my (
        $nqEntryRef, $hashRef,       $dupeList, $dupesLabel,
        $oldCallRef, $fieldTypesRef, $windowRef,
    ) = @_;

    my %config     = %{$hashRef};
    my @nqEntry    = @{$nqEntryRef};
    my @fieldNames = @{ $config{fieldNames} };
    my $window     = $$windowRef;

    print "Updating dupesheet:\n" if $config{debug};

    # Go over each entry box:
    for ( my $i = 0 ; $i <= $#nqEntry ; $i++ ) {

       # Re-write dupe sheet if field being examined is 'call' field
       # AND its contents has changed:
        return unless eval { defined $nqEntry[$i]->get };
        if (   $config{isCall} eq $fieldNames[$i]
            && $nqEntry[$i]->get ne $$oldCallRef )
        {
            if ( $nqEntry[$i]->get ne "" ) {

                # Callsign field contains something: search db:
                my $cl = $nqEntry[$i]->get;
                my $s =
                  "SELECT  \*, rowid FROM \"$config{tableName}\"
                         WHERE $config{isCall} like \'$cl\%\' ORDER
                         BY $config{dupeOrderBy}";
                ( undef, undef ) = search_and_display(
                    $dupeList,
                    undef,
                    undef,
                    $dupesLabel,
                    $s,
                    undef,
                    undef,
                    $hashRef,
                    \@{ $config{dupeShowFields} },
                    "- Dupe Sheet - Found",
                    \@nqEntry,
                    $window,
                );
                $$oldCallRef = $nqEntry[$i]->get;
            }
            else {

                # No call or partial call entered.
                # Wipe dupe sheet:
                $dupeList->delete('all');
                $dupesLabel->configure( -text => "- Dupe Sheet -" );
                $nqEntry[$i]
                  ->configure( -background => $config{entry_bg} );
                $$oldCallRef = "WIPED";
            }
        }
    }
}    # End of subroutine update_dupesheet

######################################################################
#
# subroutine refresh_field
#
# Purpose: Puts entry box in 'normal' state.
#          Deletes contents.
#          Writes new contents.
#          Pts entry box back in original state.
#
# Args:
#
#   1. Instance of the entry box $e.
#   2. New contents of the box, or value $val.
#
# Returns:
#
#      void.
#
######################################################################
#
sub refresh_field {
    my ( $e, $val ) = @_;
    eval { defined $e->get; } or return;
    my $state = $e->cget('-state') or return;
    $e->configure( -state => 'normal' );
    my $l = length( $e->get() );
    $e->delete( 0, $l );
    $e->insert( 'end', "$val" );
    $e->configure( -state => $state );
}

######################################################################
#
# subroutine change_rigtype
#
# Purpose: Goes through the array fieldTypes and removes elements of
#          type "RIG", replacing it with "CAR". Justification: If
#          there isn't a working rig, then we don't want the option
#          "RIG" to appear.
#
# Args:
#
#   Reference to hash that contains the config variables.
#
# Returns:
#
#   Undef.
#
######################################################################
#
sub change_rigtype {

    my ($hashRef) = @_;

    my %config = %{$hashRef};
    my @fT     = @{ $config{fieldTypes} };
    my @rD     = @{ $config{rigData} };
    my @fN     = @{ $config{fieldNames} };

    for ( my $i = 0 ; $i < $#fT ; $i++ ) {
        if ( $fT[$i] eq "RIG" && $rD[$i] == 1 ) {
            ${ $$hashRef{fieldTypes} }[$i] = "CAR";
            my $z = "Replacing fieldType \"RIG\" with \"CAR\" in ";
            $z = $z . "field $fN[$i] \n";
            print "$z" if $config{debug};
        }
    }
    return;
}

######################################################################
#
# subroutine get_rig_data
#
# Purpose: grabs current QRG in kHz and mode from the Hamlib 'rigctld'
# telnet server.
#
# Args:
#
#   1. Instance of newly opened rigctld IP socket
#   2. Variable indicating (if '1') that debug messages are wanted.
#
# Returns:
#
#   1. Current QRG in kHz, as a string. May be void if no
#      meaningful reply was received from 'rigctld'.
#   2. Current mode (FM/CW/SSB etc) as a string. May be void if no
#      meaningful reply was received from 'rigctld'.
#   3. String indicating there was a problem with getting data from
#      the rig. This string is normally void.
#
####################################################################
#
sub get_rig_data {

    my ( $rigctldSocket, $debug ) = @_;

    my $khz = "";
    my $mode;
    my $freq;
    my $ifWidth;
    my $rigWarning;

    die "Handle $rigctldSocket produced an error\n"
      if $rigctldSocket->error;

    #    print "Getting rig mode:\n" if $debug;
    #    if (! (print $rigctldSocket "\\get_mode\n" )) {
    #      $$rigErrCountRef++;
    #      return ('','', "Could not request mode from \'rigctld\'\n");
    #    }

    $t = "Rig reply is taking too long. Perhaps the ";
    $t = $t . "rig is off-line.\nAlso, check that the ";
    $t = $t . "\'rigctld\' process is running.\n";
    $t = $t . "If you don\'t want rig control, set ";
    $t = $t . " \'useRig=0\' in the configuration file.";

    my $n = "Time-out during get_rig_data() getting mode.\n" . "$t";
    $rigWarning = "";
    $mode       = "";
    $ifWidth      = "";
    eval {
        local $SIG{ALRM} = sub {
	  # time-out after one second to prevent lock-up.
	  die "$n";
	};
        $rigWarning = "$n";
        alarm 1; # time-out value in seconds
        my $x = "";
        do {

            # Wait for mode string. First mode, then IF width:
            while ( !( $x = $rigctldSocket->getline ) ) { ; }
            chomp $x;
            print "get_rig_data(): in get_mode loop: \$x=$x\n"
	      if $debug;
            $mode = $x 
	      if ( $x =~ /CW|USB|LSB|FM|AM|DATA|PSK|RTTY/ );
            if ( $x =~ /^\d+$/ && $mode ne "" ) {
                $ifWidth = $x if $x =~ /^\d+$/;
            }

        } until ( $ifWidth ne "" || $x =~ /RPRT/ );
        alarm 0;
        if ($ifWidth ne "") {$rigWarning = "";} 
	else {$rigWarning =  "$rigWarning" . " \$x=$x\n";}
    };

    if ( $rigWarning ne "" ) {
        $$rigErrCount++;
        return ( "", $mode, $rigWarning );
    }

    print "Requesting rig mode:\n" if $debug;
    if ( !( print $rigctldSocket "\\get_mode\n" ) ) {
        $$rigErrCountRef++;
        return ( '', '',
            "Could not request mode from \'rigctld\'\n" );
    }

    $n = "Time-out during get_rig_data() getting freq.\n" . "$t";
    $rigWarning = "";
    eval {
        local $SIG{ALRM} = sub { die "$n"; };
        $rigWarning = "$n";
        alarm 1;
        my $x = "";
        $freq = "";
        do {

            # Wait for freq data
            while ( !( $x = $rigctldSocket->getline ) ) { ; }
            chomp $x;
            print "In get_freq loop: \$x=$x\n" if $debug;
            $freq = $x if ( $x =~ /^\d+$/ );
        } until ( $freq ne "" || $x =~ /RPRT/ );
        alarm 0;
        if ($freq ne "") {$rigWarning = "";} 
	else {$rigWarning =  "$rigWarning" . " \$x=$x\n";} 
    };
    if ( $rigWarning ne "" ) {
        $$rigErrCount++;
        return ( $freq, $mode, $rigWarning );
    }

    #    return ( $freq, $mode, $rigWarning ) if $rigWarning ne "";
    #    if ($rigWarning ne ""){
    #      die "Reading freq:  $freq, $mode, $rigWarning\n";
    #    }

    if ( $freq =~ /\d+/ && $freq != 0 ) {
        if ( my $y = $freq / 1000 ) {    # convert Hz to kHz
            $khz =
              sprintf( "%.0f", $y );  # convert float khz to int khz
        }
        else {
            print "Could not convert $freq to kHz\n"
              if $config{debug};
            $khz = '';
        }
    }
    else { $khz = ''; }

    if ( $config{debug} ) {
        print "\$freq=$freq, \$mode=$mode, \$ifWidth=$ifWidth, ";
        print "\$rigWarning=$rigWarning.\n";
    }

    print "Requesting rig freq:\n" if $debug;
    if ( !( print $rigctldSocket "\\get_freq\n" ) ) {
        $$rigErrCountRef++;
        return ( '', '',
            "Could not request freq from \'rigctld\'\n" );
    }

    return ( $freq, $mode, $rigWarning ) if $rigWarning ne "";
    if ( $rigWarning ne "" ) {
        die "Reading freq:  $freq, $mode, $rigWarning\n";
    }

    return ( $khz, $mode, $rigWarning );
}

######################################################################
#
# subroutine set_rig_data
#
# Purpose: sends QRG in kHz or mode to rig.
#
# Args:
#
#   1. Instance of newly opened rigctld socket
#   3. Variable indicating (if '1') that rig control is not working
#      or wanted.
#   3. Variable indicating (if '1') that debug messages are wanted.
#   4. Variable containing QRG or mode
#   5. Instance of button which clears all fields
#
# Returns:
#
#   1. String indicating there was a problem with getting data from
#      the rig.
#
######################################################################
#
sub set_rig_data {

    my ( $rigctldSocket, $noRig, $debug, $data, $clearButton ) = @_;
    my $rigWarning = "";

    # print "Setting QRG:\n";
    if ( $data =~ /^\d+$/ ) {
        return (1) if $noRig == 1;
        my $l = length("$data");
        $data = $data . "000"    if ( ( $l == 4 ) || ( $l == 5 ) );
        $data = $data . "000000" if ( ( $l == 1 ) || ( $l == 2 ) );
        print "Setting QRG to \'$data\'...\n" if $debug;
        print $rigctldSocket "\\set_freq $data\n"
          || die "Could not write to \$rigctld\n";
        $clearButton->flash;
        $clearButton->invoke();
        return (1);
    }

    # print "Setting mode:\n";
    $data = uc($data);
    if ( $data =~ /^(USB|LSB|CW|AM|FM)$/ ) {
        return (1) if $noRig == 1;
        print "Setting mode to \'$data\'...\n" if $debug;
        print $rigctldSocket "\\set_mode $data 0\n"
          || die "Could not write to \$rigctld\n";
        $clearButton->flash;
        $clearButton->invoke();
        return (1);
    }

    return (0);
}

####################################################################
#
# subroutine grab_utc
#
# Purpose: grabs current time from system clock and returns it as
# list of dd/mm/yyyy/utc
#
# Args:
#
#   None.
#
# Returns:
#
#   1. Day, e.g. '31' (for the 31st).
#   2. Month, e.g. '12' (for December).
#   3. Year, e.g. '2005'.
#   4. Time (as UTC), e.g. '2359'.
#
####################################################################
#
sub grab_utc {

    # Update date and time:
    #
    my (
        $sec,  $min,  $hour, $mday, $mon,
        $year, $wday, $yday, $isdst
      )
      = gmtime( time() )
      or die "Fatal error: gmtime() function call failed.\n$!\n";

    #
    my $dd = $mday;

    # $mon  starts at 0,    we want to start at 1:
    my $mm = $mon + 1;

    # $year starts at 1900, we want to start at 0:
    my $yyyy = $year + 1900;
    my $utc  = ( 100 * $hour ) + $min;

    #
    # Pad with leading zeros:
    #
    $dd = '0' . $dd if length($dd) == 1;
    $mm = '0' . $mm if length($mm) == 1;
    my $c;
    for ( $c = length($utc) ; $c < 4 ; $c++ ) {
        $utc = "0" . $utc;
    }

    return ( $dd, $mm, $yyyy, $utc );
}

######################################################################
#
# callback save_new_qso
#
# Purpose: saves newly entered QSO to SQLite database.
#
# Args:
#
#   1. Reference to hash that contains new QSO data.
#   2. Instance of the frame holding the new QSO template.
#   3. Reference to array containing instances of entry boxes.
#      These are the entry boxes that we are typing new QSO data into.
#   4. Instance of a button that performs the log search function.
#   5. Instance of the main application window.
#   6. Reference to array containing the type of each field (e.g.
#      'UTC', 'RIG', '+1' etc).
#   7. Reference to hash containing config file data.
#
# Returns: void
#
######################################################################
#
sub save_new_qso {

    my ( $nqTableRef, $nqFrame, $nqEntryRef, $searchButton, $window,
        $fieldTypesRef, $hashRef, )
      = @_;

    # Create local copies:
    my %nqTable = %{$nqTableRef};
    my %config  = %{$hashRef};

    my @mustHave = @{ $config{mustHave} };

    my @nqEntry  = @{$nqEntryRef};
    my @nqFields = $config{nqFields};

    my @fieldDescr = @{ $config{fieldDescr} };
    my @fieldNames = @{ $config{fieldNames} };

    # Make sure all compulsory fields have a value:
    for ( my $i = 0 ; $i <= $#fieldNames ; $i++ ) {
        if ( $mustHave[$i] == 1 ) {
            if ( !defined( $nqTable{ $fieldNames[$i] }->get )
                || $nqTable{ $fieldNames[$i] }->get eq "" )
            {
                print "$fieldDescr[$i] required, QSO NOT logged.\n"
                  if $config{debug};

                if ( $config{typoDialog} == 1 ) {

                    # Complain loudly && do not log:
                    my_dialog(
                        $hashRef,
                        $window,
                        "Warning",
                        "Close",
                        $config{dialogGeometry},
                        " QSO not saved ",
                        "$fieldDescr[$i] required",
                    );
                }
                return (0);
            }
        }
    }

    print "\nAttempting to save QSO:\n" if $config{debug};

    # Construct SQL update statement in $s:
    my $s = "INSERT INTO \'$config{tableName}\' (";

    # Construct COLS list of statement:
    foreach my $fname ( keys(%nqTable) ) {

        # Add field name to COLS list
        $s = $s . "$fname, ";
    }

    #
    $s =~ s/\, $/) VALUES (/;

    #
    # Construct VALUES list:
    #
    foreach my $fName ( keys(%nqTable) ) {

        #
        if ( $nqTable{$fName}->get() ne "" ) {
            $tmp = $nqTable{$fName}->get();
            $tmp =~ s/'/''/g;
            $s = $s . "'" . $tmp . "', ";
        }
        else {

   # field has been left empty by the user.
   # Instead of leaving it <undef>, we put it in as an empty string.
   # Some DBD's don't like INSERTing or SELECTing records containing
   # <undef> values.
            $s = $s . "\'\', ";
        }
    }

    # Get rid of trailing comma:
    $s =~ s/\, $/) /;

    print "SQL statement: $s\n" if $config{debug};

    if ( do_SQL( $s, $hashRef ) != 1 ) {
        my $b = "Error: failed to update record in database.";
        my $a = "Failed SQL statement: $s";
        die "$a\n$b\n$!\n";
    }

    print "QSO saved.\n" if $config{debug};

    # Load default values (unless locked):
    # Clear entries.
    # Putting back UTC and RIG values is done by subroutine
    # update_entries()

    my @defaultValues = @{ $config{defaultValues} };

    # Go over all fields:
    for ( my $i = 0 ; $i <= $#nqEntry ; $i++ ) {
        if ( $config{defaultSerialField} eq $fieldNames[$i] ) {

            ##### Special case: serial number field #####

      # Inherit recently logged value and make it default if not "":
      # TODO: check for non-numerical values
            $defaultValues[$i] = $nqEntry[$i]->get
              if $nqEntry[$i]->get() ne "";

            # Increment serial number when contesting:
            if ( ${$fieldTypesRef}[$i] eq "+1" ) {
                print " Incrementing $defaultValues[$i] to "
                  if $config{debug};
                $defaultValues[$i]++
                  if ${$fieldTypesRef}[$i] eq "+1";
                print "$defaultValues[$i] \n" if $config{debug};
            }

          # Save incremented serial to both %config as defaultValue.
          # We want the incremented serial to become the new default
          # value, and also the 'locked' value.
            ${ $$hashRef{defaultValues} }[$i] = $defaultValues[$i];
        }

        # Unlock entry box and wipe contents:
        my $oldState = $nqEntry[$i]->cget( -state );
        my $oldValue = $nqEntry[$i]->get();
        my $l        = length( $nqEntry[$i]->get() );
        $nqEntry[$i]->configure( -state => 'normal' );
        $nqEntry[$i]->delete( 0, $l );

        # Pre-load entry boxes with default values:
        if ( ${$fieldTypesRef}[$i] =~ /(---|\+1)/ ) {

            # The entry box is neither locked nor carry-over.
            # Load the default field value if any:
            $nqEntry[$i]->configure( -state => 'normal' );
            if ( $defaultValues[$i] ne "" ) {
                $nqEntry[$i]->insert( 0, $defaultValues[$i] );
            }
        }

        if ( ${$fieldTypesRef}[$i] eq "LCK" ) {

            # The field is 'locked'.
            # Put locked data in entry box:
            $nqEntry[$i]->configure( -state => 'normal' );
            $nqEntry[$i]->insert( 0, $oldValue );
        }

        if ( ${$fieldTypesRef}[$i] eq "CAR" ) {

            # The field type is 'carry-over'.
            # Grab just-saved value and recycle it:
            $nqEntry[$i]->configure( -state => 'normal' );
            $nqEntry[$i]->insert( 0, $oldValue );
        }

        if ( ${$fieldTypesRef}[$i] eq "UTC" ) {

            # The field type is utc or dd, mm, yyyy.
            # Grab just-saved value and recycle it.
            # Why not just retrieve new values? Because it
            # slows down the event loop too much :-(
            $nqEntry[$i]->configure( -state => 'normal' );
            $nqEntry[$i]->insert( 0, $oldValue );
        }

        if ( ${$fieldTypesRef}[$i] eq "RIG" ) {

            # The field type is rig data.
            # Grab just-saved value and recycle it.
            # Why not just retrieve new values? Because it
            # slows down the event loop too much :-(
            $nqEntry[$i]->configure( -state => 'normal' );
            $nqEntry[$i]->insert( 0, $oldValue );
        }

        # Restore previous state of the entry box:
        $nqEntry[$i]->configure( -state => $oldState );
        $nqEntry[$i]->icursor('end');
    }

    # Refresh hList log:
    $searchButton->eventGenerate('<Enter>');
    $searchButton->eventGenerate('<ButtonPress-1>');
    $searchButton->eventGenerate('<ButtonRelease-1>');
    $searchButton->eventGenerate('<Leave>');

    # Put focus on field of choice (defaultEntryField):
    for ( my $i = 0 ; $i <= $#fieldNames ; $i++ ) {
        $nqEntry[$i]->focus
          if $fieldNames[$i] eq $config{defaultEntryField};
    }

    # return (1);
}

######################################################################
#
# subroutine select_search_method
#
# Purpose: establish whether the user wants to 'list most recent QSOs'
#          or 'search by string'. Then pass control to the appropriate
#          subroutine.
# Args:
#
#   1. Reference to scalar which, if true, indicates the user wants a
#      log view of 'most recent' QSOs. If not true, it is assumed the
#      user wants to search the log for a particular string.
#   2. Instance of button which, if clicked, allows the user to edit
#      the QSO(s) highlighted in the log view.
#   3. See "2", but this button deletes highlighted QSO(s).
#   4. Instance of an Entry widget, in which the string to be
#      searched for is typed.
#   5. Instance of an HList widget. This widget contains a view of
#      the log, one QSO per line.
#   6. Reference to a scalar which holds the name of a field. This is
#      the field the log will be searched for. E.g: 'call'if we want
#      to search the log for a paricular callsign.
#   7. Instance of a label which will be holding the number of
#      matching QSOs found.
#   8. Reference to a scalar which, if true, indicates the user wants
#      to perform a 'strict' search: case-sensitive and no wildcards.
#      If false, a 'loose' match will be performed.
#   9. Reference to array called selectedRowID. This array holds the
#      RowID(s) of QSO(s) currently highlighted in the log view
#      window.
#  10. Instance of a button which, if clicked, will kick off a log
#      search.
#  11. Reference to a hash which contains the configuration variables.
#  12. Reference to instance of current window.
#
####################################################################
#
sub select_search_method {

    my (
        $wantsRecentRef,   $editButton,
        $deleteButton,     $entry,
        $hList,            $searchFieldRef,
        $counterLabel,     $matchTypeRef,
        $selectedRowIDRef, $searchButton,
        $hashRef,          $windowRef,
        $mostRecentSerialRef,
    ) = @_;

    my $wantsRecent = $$wantsRecentRef;
    my $matchType   = $$matchTypeRef;
    my $window      = $$windowRef;

    if ($wantsRecent) {

        my ($mostRecentSerial) = show_recent(
            $editButton,   $deleteButton, $hList,
            $counterLabel, $entry,        $selectedRowIDRef,
            $searchButton, $hashRef,      $window
        );
        $$mostRecentSerialRef = $mostRecentSerial
          if defined $mostRecentSerial;
    }
    else {
        search_by_string(
            $editButton, $deleteButton,     $entry,
            $hList,      $searchFieldRef,   $counterLabel,
            \$matchType, $selectedRowIDRef, $searchButton,
            $hashRef,    $window,
        );
    }

}

####################################################################
#
# subroutine keysym_query
#
# Purpose: shows name and numeric value of keystrokes.
#
# Call like this:
# $widget->bind('<KeyPress>' => \&keysym_query,);
#
# Args:                 None. The widget object is passed by default,
#                       when called by $widget->bind.
#
# Returns:              Void. This is a callback.
#
####################################################################
#
sub keysym_query {
    my ($widget) = @_;
    my $e = $widget->XEvent;
    my ( $keysym_text, $keysym_decimal ) = ( $e->K, $e->N );
    print "\nkeysym=$keysym_text, numeric=$keysym_decimal\n";
}

####################################################################
#
# subroutine search_by_string
#
# Purpose: searches the db for QSOs, based on a value (e.g. 'ZL2BSJ')
#          and field name (e.g. 'call'). Uses the DBI interface.
#          QSOs - if found - are listed in an HList box. A
#          pre-exisiting 'edit' button is activated if QSOs were
#          found.
#
# Args:
#
# 1.  Instance of a button. This is the edit button, which will pop up
#     the editor on the selected QSO.
# 2.  Instance of a button. This is the delete button, which will pop
#     up the delete dialog on the selected QSO.
# 3.  Instance of an entry box. This is the box that conatins the
#     search string.
# 4.  HList widget containing the QSOs.
# 5.  Reference to $searchField
# 6.  Label where the number of QSOs found will be shown.
# 7.
# 8.
# 9.  Instance of a button. This is the button that kicks off the log
#     search.
# 10. Reference to hash containing all the config file contents.
# 11. Instance of current window.
#
# Returns: Void.
#
####################################################################
#
sub search_by_string {

    my (
        $editButton,   $deleteButton,     $entry,
        $hList,        $searchFieldRef,   $counterLabel,
        $matchTypeRef, $selectedRowIDRef, $searchButton,
        $hashRef,      $window,
    ) = @_;

    use DBI;

    my %config = %{$hashRef};

    my $matchType = $$matchTypeRef;

    my $searchString = $entry->get();
    return if ( $searchString eq "" || !defined($searchString) );

    my @fieldNames = @{ $config{fieldNames} };
    my @fieldDescr = @{ $config{fieldDescr} };

    # Get fieldName from $$searchFieldRef (which is a fieldDescr):
    my $fName = "";
    my $i     = 0;
    foreach my $d (@fieldDescr) {
        $fName = $fieldNames[$i]
          if "$$searchFieldRef" eq "$d";
        $i++;
    }

    # Build SQL select statement:
    # Replace '*' with '%' to accommodate UNIX wild-card searches.
    $searchString =~ s/\*/\%/g;
    my $s;
    my $like = "LIKE ";
    $like = "= " if $matchType =~ 'strict';
    $s =
"SELECT \*, rowid FROM \"$config{tableName}\" WHERE $fName $like";
    $s = $s . "\'$searchString\' ORDER BY $config{orderBy}";

    ( undef, undef ) = search_and_display(
        $hList,                       $editButton,
        $deleteButton,                $counterLabel,
        $s,                           $selectedRowIDRef,
        $searchButton,                $hashRef,
        \@{ $config{logShowFields} }, "QSOs found",
        undef,                        $window,
    );

}

######################################################################
#
# subroutine val_entry (aka 'validate entry')
#
# Purpose: checks the character entered into an entry widget as
#          it is being typed. Disables a button (arg #1, typically
#          the search  button) if entry is empty. It makes no sense
#          to search on an empty string...
#          Anything other than a-zA-Z0-9 and '/' is considered to be
#          a typo and gets ignored. We don't want typos in the log.
#
# Args:
#
# 1. A button. This button is to be activated as soon as the entry
#    widget is no longer empty.
# 2. The parent window.
# 3. Undefined.
# 4. The proposed new string in the entry widget.
# 5. The existing (soon to be old) string in $entry.
# 6. The index number of the character, just entered into the entry
#    widget.
# 7. A number indicating Insert Mode (1) or Delete Mode (0).
#
#    Args 3,4,5,6 and 7 are passed by the -validateCommand option of
#    $entry->configure. Call this subroutine as per below:
#	  -validatecommand => [ \&val_entry, $searchButton, $window,]
#
#
######################################################################
#
sub val_entry {

    my ( $searchButton, $window, $newString, $newCharacter,
        $oldString, $charIndex, $insertMode, )
      = @_;

    if ( $insertMode == 1 || $insertMode == 8 ) {
        return (0) unless $newCharacter =~ /[A-Za-z0-9\/\*\% ]/;
    }

    if ( length($newString) > 0 ) {
        $searchButton->configure( -state => "normal" );
    }
    else {
        $searchButton->configure( -state => "disabled" );
    }
    return (1);
}

######################################################################
#
# subroutine edit_qso
#
# Purpose: Pops up a separate window. In this window, the QSO to be
#          edited is loaded. The user can make alterations to the
#          fields, and update the database or discard the changes.
#
# Args:
#          1. An HList widget. The 'selected' HList list entry is the
#             QSO to be loaded into the editor.
#          2. Instance of main window widget.
#          3. Instance of a button which causes a refresh of the log
#             view.
#          4. Reference to hash containing config file data.
#          5. Junk variable used for catching unwanted callback arg.
#
# Returns:
#          Void. Callbacks don't accept return values, this code is
#          used as both subroutine and callback.
#
######################################################################
#
sub edit_qso {

    my (
        $hList,
        $window,
        $searchButton,
        $hashRef,

        # rowID passed only if double-click event calls it:
        $junk,

    ) = @_;

    my %config = %{$hashRef};

    # Grab rowID for the QSO we want to edit.
    # And pull the QSO from the DB:
    my @rowIDs = $hList->selectionGet;
    print "rowIDs:@rowIDs\n" if $config{debug};
    unless (@rowIDs) {
        my $a = "Can't exit QSO(s) because none were selected";
        my $b = "Please select QSOs before trying to edit.";
        die "$a\n$b\n$!\n";
    }

    foreach my $rowID (@rowIDs) {
        my $sqlQuery =
          "SELECT * FROM \"$config{tableName}\" WHERE rowid=$rowID";

        my $ary_ref =
          do_select( $config{db}, $config{tableName},
            $config{userName}, $sqlQuery );

        if ( scalar( @{$ary_ref} ) ne 1 ) {
            my $a = "\n";
            my $b =
              "Selecting by rowID returned more than one record.";
            my $c = "Fatal run-time error.\n";
            die "$a\n$b\n$c\n$!\n";
        }

        @qsoToEdit = @{ ${$ary_ref}[0] };

        # Pop up the editor and put a frame in it:
        my $editWindow = $window->Toplevel();
        $editWindow->title("Edit Window");
        $editWindow->configure(
            -background => $config{mainWindow_bg},
            -relief     => 'groove',
        );

        my $frame = $editWindow->Frame;

        my @subFrame;
        my @leftSubFrame;
        my @rightSubFrame;
        my @label;
        my @editBoxAry;

        my $rowidLabel = $frame->Label(
            -text       => "row-ID: $rowID",
            -foreground => $config{rowid_fg},
            -background => $config{rowid_bg},
            -relief     => 'groove',
          )->pack(
            -expand => 1,
            -fill   => 'x'
          );

      # Create sub-frames for each field, containing a header (left)
      # and an Entry box for the field (right).
        my @fieldDescr = @{ $config{fieldDescr} };

        #    my $errorMsge  = undef;
        my $typoDialog = $config{typoDialog};
        my @ebVars;
        for ( my $i = 0 ; $i <= $#qsoToEdit ; $i++ ) {
            $subFrame[$i] = $frame->Frame(
                -relief      => 'groove',
                -borderwidth => '1'
            )->pack;

            # Set up field description labels on the left:
            $label[$i] = $subFrame[$i]->Label(
                -text       => "$fieldDescr[$i]",
                -anchor     => 'w',
                -width      => '10',
                -foreground => $config{labels_fg},
                -background => $config{labels_bg},
              )->pack(
                -side   => 'left',
                -expand => 1,
                -fill   => 'both',
              );

            # Set up field value entry boxes on the right:
            $editBoxAry[$i] =
              $subFrame[$i]->Entry( -width => '15', )->pack(
                -side   => 'right',
                -expand => 1,
                -fill   => 'both',
              );

            $editBoxAry[$i]
              ->configure( -textvariable => \$ebVars[$i], );

            $editBoxAry[$i]->configure( -validate => 'key', );

            $editBoxAry[$i]->configure(
                -validatecommand => [
                    \&check_entry,   undef,
                    $hashRef,        $i,
                    $editBoxAry[$i], \$ebVars[$i]
                ],
            );

            # Catch fields that are undef:
            $ebVars[$i] = $qsoToEdit[$i]
              if defined( $qsoToEdit[$i] );

        }

        @cfmQsl = @{ $config{cfmQsl} };
        my $QslButton = $frame->Button(
            -underline        => '0',
            -activeforeground => $config{buttons_activefg},
            -activebackground => $config{buttons_activebg},
            -foreground       => $config{buttons_fg},
            -background       => $config{buttons_bg},
            -text             => "QSL received and sent",
            -command          => sub {
                for ( my $a = 0 ; $a < $#fieldDescr ; $a++ ) {
                    foreach my $b (@cfmQsl) {
                        if ( $fieldDescr[$a] eq $b ) {
                            $ebVars[$a] = $config{cfmQslValue};
                        }
                    }
                }
            },
          )->pack(
            -expand => 1,
            -fill   => 'x',
          );

        my $saveButton = $frame->Button(
            -underline        => '0',
            -activeforeground => $config{buttons_activefg},
            -activebackground => $config{buttons_activebg},
            -foreground       => $config{buttons_fg},
            -background       => $config{buttons_bg},
            -text             => "Save changes",
            -command          => [
                \&save_altered_qso, \@editBoxAry,
                $rowID,             $editWindow,
                $searchButton,      $hashRef,
            ]
          )->pack(
            -expand => 1,
            -fill   => 'x',
          );

        my $discardButton = $frame->Button(
            -underline        => '3',
            -activeforeground => $config{buttons_activefg},
            -activebackground => $config{buttons_activebg},
            -foreground       => $config{buttons_fg},
            -background       => $config{buttons_bg},
            -text             => "Discard changes",
            -command          => sub { $editWindow->destroy },
          )->pack(
            -expand => 1,
            -fill   => 'x',
          );

        # Alt-s|S generates a $saveButton event:
        $editWindow->bind(
            '<Alt-s>' => sub {
                $saveButton->invoke();
            }
        );

        $editWindow->bind(
            '<Alt-S>' => sub {
                $saveButton->invoke();
            }
        );

        # Alt-c|C generates a $discardButton event:
        $editWindow->bind(
            '<Alt-c>' => sub {
                $discardButton->invoke();
            }
        );

        $editWindow->bind(
            '<Alt-C>' => sub {
                $discardButton->invoke();
            }
        );

        # Escape generates a $discardButton event:
        $editWindow->bind(
            '<Escape>' => sub {
                $discardButton->invoke();
            }
        );

        # Alt-q|Q generates a $QslButton event:
        $editWindow->bind(
            '<Alt-q>' => sub {
                $QslButton->invoke();
            }
        );
        $editWindow->bind(
            '<Alt-Q>' => sub {
                $QslButton->invoke();
            }
        );

        # Pack everything:
        $frame->pack( -expand => 1, -fill => 'both' );
    }
}

######################################################################
#
# subroutine cabexport_qso
#
# Purpose: Pops up a separate window. In this window, the user can
#          control how the selected QSOs are to be exported.
#
# Args:
#          1. Reference to an HList widget. The 'selected' HList list
#             entry is the QSO to be loaded into the editor.
#          2. Reference to instance of main window widget.
#          4. Reference to hash containing config file data.
#          5. Junk variable used for catching unwanted callback arg.
#
# Returns:
#          Void. Callbacks don't accept return values, this code is
#          used as both subroutine and callback.
#
####################################################################
#
sub cabexport_qso {

    my (
        $hListRef,
        $windowRef,
        $configRef,
        $progNameRef,
        $junk,

    ) = @_;

    my $progName = $$progNameRef;
    my $hList    = $$hListRef;
    my $window   = $$windowRef;
    my %config   = %{$configRef};

    # Grab rowIDs for the QSOs we want to export:

    my @rowIDs = $hList->selectionGet;
    print "rowIDs:@rowIDs\n" if $config{debug};
    unless (@rowIDs) {
        my $a = "No QSOs selected.\n";
        my $b = "Please select QSO(s) before trying to export.\n";
        my_dialog( $configRef, $$windowRef, "Warning", "Close",
            $config{dialogGeometry}, "$a$b" );
        goto END;
    }

    # Reverse array @rowIDs so the Cabrillo dump is in
    # chronological order:
    reverse_array( \@rowIDs );

    # Pop up the export window && populate:
    my $exportWindow = $window->Toplevel(
        -title       => "Export Window",
        -relief      => 'raised',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    );

    $exportWindow->geometry( $config{exportGeometry} );

    my $n            = $#rowIDs + 1;
    my $cancelButton = $exportWindow->Button(
        -underline        => '0',
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -text             => "Cancel",
        -command          => sub { $exportWindow->destroy },
      )->pack(
        -expand => 0,
        -fill   => 'x',
        -side   => 'top',
        -anchor => 'n',
      );

    # Building label text for Cabrillo export window:
    my $q = "QSOs";
    $q = "QSO" if $n == 1;
    my $pwd = `pwd`;
    chomp($pwd);
    my $cxf = "$pwd" . '/' . "$config{cabrilloExportFileName}";
    my $ewt =
      "Exporting $n selected $q to Cabrillo file:\n$cxf\n\n";
    $ewt = $ewt . "Existing data will be overwritten.\n\n";
    $ewt =
      $ewt . "The following \'cabrilloExport\' parameters will\n";
    $ewt = $ewt . "be used for the conversion:\n\n";

    foreach my $c ( @{ $config{cabrilloExport} } ) {
        $ewt = $ewt . "$c\n";
    }
    my $ewLabelTxt = $ewt;

    my $ewLabel = $exportWindow->Label(
        -text       => $ewLabelTxt,
        -foreground => $config{labels_fg},
        -background => $config{labels_bg},
    )->pack( -expand => 1, -fill => 'both', -side => 'top', );

    my $exportMenubutton = $exportWindow->Button(
        -underline        => '0',
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -text             => "Export",
      )->pack(
        -expand => 0,
        -fill   => 'x',
        -side   => 'bottom',
        -anchor => 's',
      );

    $exportMenubutton->configure(
        -command => [
            \&write_cabexport,  \$exportWindow,
            \%config,           \@rowIDs,
            \$exportMenubutton, \$cancelButton,
            \$ewLabel,          \$progName,
        ],
    );

    # Alt-e|E generates a $exportMenubutton event:
    $exportWindow->bind(
        '<Alt-e>' => sub {
            $exportMenubutton->invoke();
        }
    );

    $exportWindow->bind(
        '<Alt-E>' => sub {
            $exportMenubutton->invoke();
        }
    );

    # Alt-c|C generates a $cancelButton event:
    $exportWindow->bind(
        '<Alt-c>' => sub {
            $cancelButton->invoke();
        }
    );

    $exportWindow->bind(
        '<Alt-C>' => sub {
            $cancelButton->invoke();
        }
    );

    # Escape generates a $cancelButton event:
    $exportWindow->bind(
        '<Escape>' => sub {
            $cancelButton->invoke();
        }
    );

  END:
}

######################################################################
#
# callback write_cabexport
#
# Purpose: Formats and writes QSOs indicated by $rowIDsRef to
#          cabrillo log.
#
# Args:
#          1. Instance of a window (the export window)
#          2. Reference to hash containing config.
#          3. Reference to array containing rowIDs of the QSOs to be
#             exported.
#          4. Reference to instance of the button that kicks off the
#             export.
#          5. Reference to instance of the button that cancels the
#             export.
#          7. Reference to scalar containing name/version of program.
#          4. Reference to instance of a label.
#          8. junk variable to mop up spurious callback args.
#
# Returns:    Void.
#
######################################################################
#
sub write_cabexport {

    my ( $exportWindowRef, $configRef, $rowIDsRef,
        $exportMenubuttonRef, $cancelButtonRef, $ewLabelRef,
        $progNameRef, $junk )
      = @_;

    my $progName = $$progNameRef;
    $progName =~ s/.pl$//;
    my %config = %{$configRef};
    my @rowIDs = @{$rowIDsRef};

    # Check output file:
    open( CABRILLO, '>', $config{cabrilloExportFileName} )
      or die
"Could not open $config{cabrilloExportFileName}\n for writing.";

    $$exportMenubuttonRef->destroy;
    $$ewLabelRef->destroy;
    $$cancelButtonRef->configure( -text => 'Close' );

    my $ewText = $$exportWindowRef->Scrolled("Text");
    $ewText->configure(
        -foreground       => $config{cabrilloBody_fg},
        -background       => $config{cabrilloBody_bg},
        -selectforeground => $config{hList_sel_fg},
        -selectbackground => $config{hList_sel_bg},
        -wrap             => 'none',
    );

    $ewText->pack(
        -expand => 1,
        -fill   => 'both',
        -side   => 'bottom',
        -anchor => 's',
    );

    foreach my $hdr ( @{ $config{cabrilloExportHeader} } ) {

        # Special case: add version number of prog to 'CREATED-BY:':
        $hdr = "$hdr" . " $progName" if ( $hdr =~ m/CREATED-BY/ );
        print CABRILLO "$hdr\n";
        $ewText->insert( 'end', "$hdr\n", 'hdr' );
    }
    $ewText->tagConfigure( 'hdr', 'foreground',
        $config{cabrilloHeader_fg} );

  # Build list of fieldNames that we want to extract for the purpose
  # of exporting to Cabrillo:
    my $selectItems = "";
    foreach my $cex ( @{ $config{cabrilloExport} } ) {
        my ( $ce, $junk ) =
          split( /:/, $cex );    # Discard optional format spec
        foreach my $fn ( @{ $config{fieldNames} } ) {

          # The cabrilloExport item is added to the 'select' list if
          # it also appears in the array of fieldNames.
            $selectItems = $selectItems . " $ce,"
              if ( $ce eq $fn );
        }
    }

    # Get rid of trailing comma:
    $selectItems =~ s/,$//;
    print "\$selectItems=$selectItems\n" if $config{debug} == 1;

    # Retrieve QSOs, one at a time and process:
    foreach my $rowID (@rowIDs) {
        my $sqlQuery =
"SELECT $selectItems  FROM \"$config{tableName}\" WHERE rowid=$rowID";

        my $ary_ref =
          do_select( $config{db}, $config{tableName},
            $config{userName}, $sqlQuery );

        if ( scalar( @{$ary_ref} ) ne 1 ) {
            my $a = "\n";
            my $b =
              "Selecting by rowID returned more than one record.";
            my $c = "Fatal run-time error.\n";
            die "$a\n$b\n$c\n$!\n";
        }

        my $outputLine  = "QSO: ";
        my @qsoToExport = @{ ${$ary_ref}[0] };
        my $i           = 0;

      CE: foreach my $cex ( @{ $config{cabrilloExport} } ) {

            # Parse the cabrilloExport statement:

            my $spaces = "";
            my $digits = "";
            my $align  = "";

            # Look for left-hand alignment symbol '-':
            if ( $cex =~ m/\-/ ) {
                $align = "-";
            }

            # Give $ce a starting value. This will be the working
            # value unless there is a format specifier involved
            # as per below (three possiblities):
            my $ce = $cex;

            if ( $cex =~ m/.+:-?\d{1,2}d$/ ) {

           # cabrilloExport statement looks e.g like this: 'utc:12d'
           # pad out to 12 digits with zeros:
                ( $ce, $digits ) = split( /:/, $cex );

                # Remove the 'd' from the $digits scalar (we only
                # want the number)
                $digits =~ s/d//;

                # Remove left-hand alignment indicator from $digits:
                $digits =~ s/-//;

            }

            if ( $cex =~ m/.+:-?\d{1,2}s$/ ) {

            # cabrilloExport statement looks e.g like this: 'utc:4s'
            # pad out to 4 spaces:
                ( $ce, $spaces ) = split( /:/, $cex );

                # We only want the number so get rid of the 's'
                $spaces =~ s/s//;

                # ...and also get rid of a possible left-hand
                # alignment symbol '-'
                $spaces =~ s/-//;

                # print "cex=$cex, ce=$ce, spaces=$spaces\n";
            }

            if ( $cex =~ m/.+:$/ ) {

              # cabrilloExport statement looks e.g like this: 'utc:'
              # so get rid of trailing ':'
                $ce = $cex;
                $ce =~ s/:$//;
            }

            my $isFieldName = 0;
          FN: foreach my $fn ( @{ $config{fieldNames} } ) {
                my $tail = " ";
                if ( $ce eq $fn ) {

                  # The cabrilloExport string is a field name, so it
                  # must have been retrieved. We can pull it out of
                  # the array and process it:
                    $isFieldName = 1;
                    $tail        = "-" if $fn eq $config{isYear};
                    $tail        = "-" if $fn eq $config{isMonth};
                    if ( $digits ne "" ) {
                        $fmt =
                          "%" . "$align" . "0" . "$digits" . "d";
                        my $formatted = sprintf "$fmt",
                          $qsoToExport[ $i++ ];
                        $outputLine =
                          $outputLine . "$formatted" . "$tail";
                        $digits = "";
                        next FN;
                    }

                    if ( $spaces ne "" ) {
                        $fmt = "%" . "$align" . "$spaces" . "s";

                        # print "fmt=$fmt\n";
                        my $formatted = sprintf "$fmt",
                          $qsoToExport[ $i++ ];

                        # print "formatted=$formatted\n";
                        $outputLine =
                          $outputLine . "$formatted" . "$tail";

                        # print "outputLine=$outputLine\n";
                        $spaces = "";
                        next FN;
                    }
                    $outputLine =
                      $outputLine . "$qsoToExport[$i++]" . "$tail";
                }
            }
            if ( $isFieldName == 0 ) {
                $outputLine = $outputLine . "$ce" . " ";
                next CE;
            }
        }

        # Clean up trailing whitespaces:
        $outputLine =~ s/ +$//;
        $ewText->insert( 'end', "$outputLine\n" );
        print CABRILLO "$outputLine\n";
        print "$outputLine\n" if $config{debug} == 1;
    }
    print CABRILLO "$config{cabrilloExportFooter}\n";
    $ewText->insert( 'end', "$config{cabrilloExportFooter}\n",
        'footer' );
    $ewText->tagConfigure( 'footer', 'foreground',
        $config{cabrilloFooter_fg} );
    $ewText->configure( -state => 'disabled', );

    close(CABRILLO);

}    # end of subroute write_cabexport() - dodgy stuff, eh!

######################################################################
#
# subroutine adifexport_qso
#
# Purpose: Pops up a separate window. In this window, the user can
#          control how the selected QSOs are to be exported.
#
# Args:
#          1. Reference to an HList widget. The 'selected' HList list
#             entries are the QSOs to be exported.
#          2. Reference to instance of main window widget.
#          4. Reference to hash containing config file data.
#          5. Junk variable used for catching unwanted callback arg.
#
# Returns:
#          Void. Callbacks don't produce return values, this code is
#          used as both subroutine and callback.
#
######################################################################
#
sub adifexport_qso {

    my (
        $hListRef,
        $windowRef,
        $configRef,
        $junk,

    ) = @_;

    my $hList  = $$hListRef;
    my $window = $$windowRef;
    my %config = %{$configRef};

    foreach my $w ( @{ $config{adifMap} } ) {
        print "adifMap: $w\n";
    }

    # Grab rowIDs for the QSOs we want to export:

    my @rowIDs = $hList->selectionGet;

    # Reverse array @rowIDs so the ADIF file is in chronological
    # order:
    reverse_array( \@rowIDs );

    print "rowIDs:@rowIDs\n" if $config{debug};
    unless (@rowIDs) {
        my $a = "No QSOs selected.\n";
        my $b = "Please select QSO(s) before trying to export.\n";
        my_dialog( $configRef, $$windowRef, "Warning", "Close",
            $config{dialogGeometry}, "$a$b" );
        goto END;
    }

    # Pop up the export window && populate:
    my $exportWindow = $window->Toplevel(
        -title       => "Export Window",
        -relief      => 'raised',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    );

    $exportWindow->geometry( $config{exportGeometry} );

    my $n            = $#rowIDs + 1;
    my $cancelButton = $exportWindow->Button(
        -underline        => '0',
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -text             => "Cancel",
        -command          => sub { $exportWindow->destroy },
      )->pack(
        -expand => 0,
        -fill   => 'x',
        -side   => 'top',
        -anchor => 'n',
      );

    # Building label text for ADIF export window:
    my $q = "QSOs";
    $q = "QSO" if $n == 1;
    my $pwd = `pwd`;
    chomp($pwd);
    my $cxf = "$pwd" . '/' . "$config{adifExportFileName}";
    my $ewt = "Exporting $n selected $q to ADIF file:\n$cxf\n\n";
    $ewt = $ewt . "Existing data will be overwritten.\n\n";
    $ewt = $ewt . "The following \'adifMap\' parameters will\n";
    $ewt = $ewt . "be used for the conversion:\n\n";

    foreach my $c ( @{ $config{adifExport} } ) {
        $ewt = $ewt . "$c\n";
    }
    my $ewLabelTxt = $ewt;

    my $ewLabel = $exportWindow->Label(
        -text       => $ewLabelTxt,
        -foreground => $config{labels_fg},
        -background => $config{labels_bg},
    )->pack( -expand => 1, -fill => 'both', -side => 'top', );

    my $exportMenubutton = $exportWindow->Button(
        -underline        => '0',
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -text             => "Export",
      )->pack(
        -expand => 0,
        -fill   => 'x',
        -side   => 'bottom',
        -anchor => 's',
      );

    $exportMenubutton->configure(
        -command => [
            \&write_adifexport, \$exportWindow,
            \%config,           \@rowIDs,
            \$exportMenubutton, \$cancelButton,
            \$ewLabel,
        ],
    );

    # Alt-e|E generates a $exportMenubutton event:
    $exportWindow->bind(
        '<Alt-e>' => sub {
            $exportMenubutton->invoke();
        }
    );

    $exportWindow->bind(
        '<Alt-E>' => sub {
            $exportMenubutton->invoke();
        }
    );

    # Alt-c|C generates a $cancelButton event:
    $exportWindow->bind(
        '<Alt-c>' => sub {
            $cancelButton->invoke();
        }
    );

    $exportWindow->bind(
        '<Alt-C>' => sub {
            $cancelButton->invoke();
        }
    );

    # Escape generates a $cancelButton event:
    $exportWindow->bind(
        '<Escape>' => sub {
            $cancelButton->invoke();
        }
    );

  END:
}

######################################################################
#
# callback write_adifexport
#
# Purpose: Formats and writes QSOs indicated by $rowIDsRef to ADIF
#          log.
#
# Args:
#          1. Instance of a window (the export window)
#          2. Reference to hash containing config.
#          3. Reference to array containing rowIDs of the QSOs to be
#             exported.
#          4. Reference to Menubutton
#          5. Reference to 'cancel' button in export window.
#          6. Reference to Label in export window.
#          7. Catch-all junk scalar.
#
# Returns:    Void.
#
####################################################################
#
sub write_adifexport {

    my ( $exportWindowRef, $configRef, $rowIDsRef,
        $exportMenubuttonRef, $cancelButtonRef, $ewLabelRef, $junk )
      = @_;

    my %config = %{$configRef};
    my @rowIDs = @{$rowIDsRef};

    # Check output file:
    open( ADIF, '>', $config{adifExportFileName} )
      or die
      "Could not open $config{adifExportFileName}\n for writing.";

    # We want to re-use the export window, so first we need
    # to knock off some unwanted widgets:
    $$exportMenubuttonRef->destroy;
    $$ewLabelRef->destroy;

    # Now re-name the 'cancel' button, so that it can be re-used as
    # 'close':
    $$cancelButtonRef->configure( -text => 'Close' );

    # Create the Text widget, used for showing the ADIF data:
    my $ewText = $$exportWindowRef->Scrolled("Text");
    $ewText->configure(
        -foreground       => $config{adifBody_fg},
        -background       => $config{adifBody_bg},
        -selectforeground => $config{hList_sel_fg},
        -selectbackground => $config{hList_sel_bg},
        -wrap             => 'none',
    );

    $ewText->pack(
        -expand => 1,
        -fill   => 'both',
        -side   => 'bottom',
        -anchor => 's',
    );

    # Copy adifHeaders into it:
    foreach my $hdr ( @{ $config{adifExportHeader} } ) {
        print ADIF "$hdr\n";
        $ewText->insert( 'end', "$hdr\n", 'hdr' );
    }

    # Colourise headers according to adifHeader_fg:
    $ewText->tagConfigure( 'hdr', 'foreground',
        $config{adifHeader_fg} );

    # In debug mode, show adifMap:
    if ( $config{debug} ) {
        foreach my $z ( @{ $config{adifExport} } ) {
            print "adifMap: $z\n";
        }
    }

  # Build list of fieldNames that we want to extract for the purpose
  # of exporting to ADIF:
    my $selectItems = "";
    foreach my $aex ( @{ $config{adifExport} } ) {
        my ( $ae, $afn, $spec ) = split( /:/, $aex );
        die "Wrong format \'adifExport = $aex\' in config file\n"
          unless defined $ae && defined $afn;
        foreach my $fn ( @{ $config{fieldNames} } ) {

            # The adifMap item is added to the 'select' list if
            # it also appears in the array of fieldNames.
            print "\'$aex\' added \'$afn\' as \'$ae\'. \$fn=$fn.\n"
              if $afn eq $fn && $config{debug};
            $selectItems = $selectItems . " $afn,"
              if ( $afn eq $fn );
        }
    }

    # Get rid of trailing comma:
    $selectItems =~ s/,$//;
    print "\$selectItems=$selectItems\n" if $config{debug} == 1;

    # Retrieve QSOs, one at a time and process:
    foreach my $rowID (@rowIDs) {
        my $sqlQuery =
"SELECT $selectItems  FROM \"$config{tableName}\" WHERE rowid=$rowID";

        my $ary_ref =
          do_select( $config{db}, $config{tableName},
            $config{userName}, $sqlQuery );

        if ( scalar( @{$ary_ref} ) ne 1 ) {
            my $a = "\n";
            my $b =
              "Selecting by rowID returned more than one record.";
            my $c = "Fatal run-time error.\n";
            die "$a\n$b\n$c\n$!\n";
        }

        my $outputLine  = "";
        my $dd          = "";
        my $mm          = "";
        my $yyyy        = "";
        my @qsoToExport = @{ ${$ary_ref}[0] };
        my $i           = 0;

        foreach my $aex ( @{ $config{adifExport} } ) {

            # Parse the adifMap statement:
            my ( $ae, $afn, $spec ) = split( /:/, $aex );
            unless (
                (
                       defined($ae)
                    && $ae ne ""
                    && defined($afn)
                    && $afn ne ""
                )
              )
            {
                die "Error in config file:\n adifExport "
                  . "statement should look like\n"
                  . "ADIF-name:fieldName[:<spec>] in "
                  . "line \'adifExport = $aex \'\n";
                $$exportWindowRef->destroy;
            }

            my $isFieldName = 0;
            foreach my $fn ( @{ $config{fieldNames} } ) {

                if ( $afn eq $fn ) {
                    $isFieldName = 1;

                    # The adifMap string contains a valid field name
                    # We can pull it out of the array
                    # and process it normally:

                    # Some special cases:
                    if ( $fn eq $config{isDay} ) {
                        die
                          "Field $qsoToExport[$i] is wrong format. "
                          . "Expected day \'dd\'\n"
                          && $$exportWindowRef->destroy
                          unless $qsoToExport[$i] =~ m/^\d?\d$/;
                        $dd = sprintf( "%02d", $qsoToExport[$i] );
                        goto SKIP;
                    }

                    if ( $fn eq $config{isMonth} ) {
                        die
                          "Field $qsoToExport[$i] is wrong format. "
                          . "Expected month \'mm\'\n"
                          && $$exportWindowRef->destroy
                          unless $qsoToExport[$i] =~ m/^\d?\d$/;
                        $mm = sprintf( "%02d", $qsoToExport[$i] );
                        goto SKIP;
                    }

                    if ( $fn eq $config{isYear} ) {
                        die
                          "Field $qsoToExport[$i] is wrong format. "
                          . "Expected year \'yyyy\'\n"
                          && $$exportWindowRef->destroy
                          unless $qsoToExport[$i] =~ m/^\d\d\d\d$/;
                        $yyyy = sprintf( "%04d", $qsoToExport[$i] );
                        goto SKIP;
                    }

                    if ( $fn eq $config{isKhz} ) {

                        # Replace khz (freq) with band (wavelength).
                        # ADIF wants bands, not khz:
                        my $khz = $qsoToExport[$i];

                       # Strip off decimal point and trailing chars:
                        $khz =~ s/\..*$//;
                        die
"Bad frequency in field $fn: \'$khz\' must "
                          . "contain numbers only\n"
                          && $$exportWindowRef->destroy
                          unless $khz =~ m/^\d+$/;
                        $qsoToExport[$i] =
                          khz_to_meters( $configRef, $khz )
                          if uc($ae) eq 'BAND';
                        $qsoToExport[$i] = khz_to_mhz($khz)
                          if uc($ae) eq 'FREQ';
                        die
"$fn must convert to either BAND or FREQ\n"
                          unless uc($ae) =~ /FREQ|BAND/;
                    }

                    # Now add the field to the ADIF line:
                    $ae = uc($ae);
                    my $d = "";
                    $d = $qsoToExport[$i]
                      if defined $qsoToExport[$i];
                    my $fmt = "";
                    if ( defined($spec) && $spec ne "" ) {

                        # Apply formatting, if required:
                        if ( $spec =~ m/d$/ ) {
                            $fmt = "%" . "0" . "$spec"
                              if $spec =~ m/d$/;
                        }
                        else {
                            $fmt = "%" . "$spec";
                        }
                        $d = sprintf( "$fmt", $d ) if $d ne "";
                    }
                    my $dataLength = 0;
                    $dataLength = length($d) if $d ne "";
                    $outputLine =
                      $outputLine . "<$ae:$dataLength>$d "
                      unless ( $dataLength == 0 || $d eq "" );
                  SKIP:
                    $i++;
                }
            }

            if ( $isFieldName == 0 ) {

                # The item to go into the ADIF line is a constant
                # that is the same across all QSOs (e.g. your call)
                # Just print it as directed by adifExport:
                $ae = uc($ae);
                my $strLength = length($ae);
                $outputLine = $outputLine . "<$ae:$strLength>$afn "
                  unless $strLength == 0 || $afn eq "";
            }

            if ( $dd ne "" and $mm ne "" and $yyyy ne "" ) {

                # We now have a full 'yyyymmdd' date stamp.
                # Add it to the front of the ADIF line:
                my $qsoDate    = "$yyyy" . "$mm" . "$dd";
                my $dateLength = length($qsoDate);
                die("Bad date-stamp, not 8 characters long\n")
                  && $$exportWindowRef->destroy
                  unless $dateLength == 8
                      && $qsoDate =~ m/^\d{8}$/;
                $outputLine =
                  "<QSO_DATE:$dateLength>$qsoDate " . "$outputLine";
                $dd   = "";
                $mm   = "";
                $yyyy = "";
            }

        }

        # Clean up trailing whitespaces:
        $outputLine =~ s/ +$//;
        $outputLine = $outputLine . "\n<EOR>\n";

        $ewText->insert( 'end', "$outputLine" );
        print ADIF "$outputLine";
        print "$outputLine" if $config{debug} == 1;
    }
    print ADIF "$config{adifExportFooter}\n";
    $ewText->insert( 'end', "$config{adifExportFooter}\n",
        'footer' );
    $ewText->tagConfigure( 'footer', 'foreground',
        $config{adifFooter_fg} );
    $ewText->configure( -state => 'disabled', );

    close(ADIF);
    print "$config{adifExportFooter}\n" if $config{debug} == 1;

}    # end of subroute write_adifexport()

######################################################################
#
# subroutine khz_to_meters
#
# Purpose: Converts frequency in kHz to wavelength ('band') in meters.
#          Uses table 'bandLimits' in config file, e.g:
#          bandLimits = 160m:1800:1900
#
# Args:    1. Scalar containing kHz.
#
#
# Returns: String containing the band.
######################################################################
#
sub khz_to_meters {

    my ( $aryRef, $khz ) = @_;
    foreach my $bandLimit ( @{ $config{bandLimits} } ) {
        my ( $band, $lower, $upper ) = split( /:/, $bandLimit );
        return ($band) if ( $khz >= $lower && $khz < $upper );
    }
}

######################################################################
#
# subroutine khz_to_mhz
#
# Purpose: Converts frequency in kHz to frequency in MHz
#          by inserting a decimal point before the 3rd character from
#          the end.
#
# Args:    1. Scalar containing kHz.
#
#
# Returns: String containing MHz.
#
######################################################################
#
sub khz_to_mhz {
    my ($khz) = @_;
    print "khz=$khz\n";
    $khz =~ m/^(\d*)(\d\d\d)$/;
    my $r = "0";

    # Handle exceptional cases: $1 is undef and $1 is empty string.
    # In these cases: use '0' for the MHz portion.
    if ( defined $1 ) {
        $r = $1 unless $1 eq "";
    }
    $r = "$r.$2";
    return $r;
}

######################################################################
#
# callback save_altered_qso
#
# Purpose: saves newly altered QSO:
#
# Args:    1. Reference to array containing instances of entry boxes.
#             These entry boxes contain the new field values to be
#             saved.
#          2. Scalar which is the rowID of the QSO to be updated.
#          3. Instance of a window (the edit window)
#          4. Instance of a button, the searchbutton.
#          5. Reference to hash containing config.
#
# Returns:    Void.
#
######################################################################
#
sub save_altered_qso {

    my ( $aryRef, $rowID, $editWindow, $searchButton, $hashRef, ) =
      @_;

    my %config = %{$hashRef};
    my @newQso;
    my $i = 0;
    foreach my $e ( @{$aryRef} ) {
        $newQso[$i] = $e->get();
        $i++;
    }

    my @fieldNames = @{ $config{fieldNames} };

    # Construct SQL update statement in $s:
    $i = 0;

    my $s = "UPDATE \'$config{tableName}\' SET ";
    foreach my $field (@newQso) {
        $field =~ s/'/''/g;
        $s = $s . "$fieldNames[$i]='";
        $s = $s . "$field";
        $s = $s . "', ";
        $i++;
    }

    $s =~ s/, $/ /;
    $s = $s . " WHERE rowid=$rowID";

    if ( do_SQL( $s, $hashRef ) != 1 ) {
        my $a = "Error: failed to update record in database.";
        my $b = "Failed SQL statement: $s";
        die "$a\n$b\n$!\n";
    }

    # Refresh viewable log:
    refresh_list($searchButton) if defined $searchButton;

    $editWindow->destroy;
}

######################################################################
#
# subroutine refresh_list
#
# Purpose: refreshes the hList log by generating a $searchButton
#          event.
#
######################################################################
#
sub refresh_list {
    my ($searchButton) = @_;
    if ( defined $searchButton ) {
        $searchButton->eventGenerate('<Enter>');
        $searchButton->eventGenerate('<ButtonPress-1>');
        $searchButton->eventGenerate('<ButtonRelease-1>');
        $searchButton->eventGenerate('<Leave>');
    }
}

######################################################################
#
# callback delete_qso
#
# Purpose: deletes QSO in edit window. Destroys edit window.
#
# Args:    1. Instance of HList widget holding the QSOs.
#             The 'selected' ones are to be deleted.
#          2. Instance of a window holding the widgets.
#          3. Instance of the button that initiated the delete action.
#          4. Reference to config hash
#
# Returns:    Void.
#
######################################################################
#
sub delete_qso {

    my ( $hList, $window, $searchButton, $hashRef, ) = @_;

    my %config = %{$hashRef};

    my @rowIDs = $hList->selectionGet;

    print "rowIDs to be deleted: @rowIDs\n" if $config{debug};

    # Pop up an 'are you sure?' dialog:
    my $deleteWindow = $window->Toplevel(
        -title       => " Delete dialog ",
        -relief      => 'raised',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    );

    $deleteWindow->geometry( $config{dialogGeometry} );

    my $okButton = $deleteWindow->Button(
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -underline        => '0',
        -text             => "Delete QSO",
        -command          => sub {

            # Construct SQL delete statement in $s:
            foreach my $rowID (@rowIDs) {

                my $s =
"DELETE FROM \'$config{tableName}\' WHERE rowid=$rowID";

                if ( do_SQL( $s, $hashRef ) != 1 ) {
                    my $a =
"Error: failed to delete record from database.";
                    my $b = "Failed SQL statement: $s";
                    die "$a\n$b\n$!\n";
                }

            }

            # Destroy delete window:
            $deleteWindow->grabRelease;
            $deleteWindow->destroy;

            # Refresh hList log:
            refresh_list($searchButton);
        },
      )->pack(
        -expand => 1,
        -fill   => 'x',
        -side   => 'bottom',
        -anchor => 's',
      );

    $deleteWindow->bind(
        '<Alt-d>' => sub {
            $okButton->invoke();
        }
    );
    $deleteWindow->bind(
        '<Alt-D>' => sub {
            $okButton->invoke();
        }
    );

    my $cancelButton = $deleteWindow->Button(
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -underline        => '0',
        -text             => "Cancel",
        -command          => sub {

            # Destroy delete window:
            $deleteWindow->grabRelease;
            $deleteWindow->destroy;
        },
      )->pack(
        -expand => 1,
        -fill   => 'x',
        -side   => 'top',
        -anchor => 'n',
      );
    $deleteWindow->bind(
        '<Alt-C>' => sub {
            $cancelButton->invoke();
        }
    );
    $deleteWindow->bind(
        '<Alt-c>' => sub {
            $cancelButton->invoke();
        }
    );
    $deleteWindow->bind(
        '<Escape>' => sub {
            $cancelButton->invoke();
        }
    );
    $deleteWindow->Label(
        -text        => " OK to delete selected QSO(s)?",
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    )->pack( -expand => 1, -fill => 'both', -side => 'top', );

    # Always on top:
    $deleteWindow->transient( $deleteWindow->Parent->toplevel );
    $deleteWindow->grab;
    $deleteWindow->raise;
    $okButton->focus;

}

######################################################################
#
# subroutine show_recent
#
# Purpose: Extracts a certain number of the most recent QSOs from the
#          db.
#          Displays the QSOs in a pre-existing HList widget.
#          Shows the number of QSOs in the counter label widget.
#          Enables the edit botton if QSOs were retrieved, Disables
#          the button otherwise.
#
# Args:    1. 'edit' button widget (see above).
#          2. 'search' button widget.
#          3. HList widget for displaying QSOs in.
#          4. 'counter' label to show number of hits.
#          5. Instance of a window.
#
# Returns: 1. Scalar containing the latest serial nummer transmitted,
#             or, undef otherwise.
#
######################################################################
#
sub show_recent {
    my (
        $editButton,   $deleteButton, $hList,
        $counterLabel, $entry,        $selectedRowIDRef,
        $searchButton, $hashRef,      $window
    ) = @_;

    my %config = %{$hashRef};

    $entry->configure( -state => "disabled" );
    use DBI;

    # Build SQL select statement:

    my $s;
    $s =
"SELECT \*, rowid FROM  \"$config{tableName}\" ORDER BY $config{orderBy}";
    $s = $s . " LIMIT $config{rNumber}";

    my ( undef, $mostRecentSerial ) = search_and_display(
        $hList,                       $editButton,
        $deleteButton,                $counterLabel,
        $s,                           $selectedRowIDRef,
        $searchButton,                $hashRef,
        \@{ $config{logShowFields} }, "QSOs found",
        undef,                        $window,
    );

    if ( defined $mostRecentSerial ) {
        return $mostRecentSerial;
    }
    else {
        return (undef);
    }
}

######################################################################
#
# subroutine do_select
#
# Purpose: Executes a select statement. Returns a reference to an
#          array.
#
#          The elements of this array are references to other arrays.
#          These other arrays contain a record each, one field per
#          element.
#
######################################################################
#
sub do_select {
    my ( $db, $tableName, $userName, $statement, ) = @_;

    my $dbh;
    eval {
        $dbh = DBI->connect(
            "dbi:SQLite:dbname=$db",
            "", "",
            {
                RaiseError => 1,
                PrintError => 1,
                AutoCommit => 1,
            }
        );
    };

    # DBI doesn't seem to flag *any* errors at this point
    # Pointless error-trapping code follows:

    if ($@) {
        print "\n";
        print "Error: Could not connect to database $db\n";
        print "Database driver returned: $dbh->errstr \n"
          if defined($dbh);
        print "DBI\:\:connect returned undef for \$\$dbh.\n";
        die;
    }

    if ( !defined($dbh) ) {
        print "\n";
        print "Error: Could not connect to database $db\n";
        print "DBI\:\:connect returned undef for \$\$dbh.\n";
        die;
    }

    if ( !$dbh ) {
        print "\n";
        print "Error: Could not connect to database $db\n";
        print "DBI\:\:connect:  returned 'false'.\n";
        print "DBI::errstr\n";
        die;
    }

    my $ary_ref;
    eval { $ary_ref = $dbh->selectall_arrayref($statement); };

    # Trap failures:
    if ($@) {
        print "\n";
        print "Error: could not select records from database\n";
        print "SQL statement: $statement\n";
        print "Database driver returned: $dbh->errstr\n";
        print "DBI             says    : $DBI::errstr\n";
        die;
    }

    if ( !defined($ary_ref) ) {
        print "\n";
        print "Error: expression returned undef for \$ary_ref\n";
        print "SQL statement: $statement\n";
        print "Database driver returned: $dbh->errstr\n";
        print "DBI             says    : $DBI::errstr\n";
        die;
    }

    return ($ary_ref);

}

######################################################################
#
# subroutine search_and_display
#
# Purpose: Executes a select statement. Returns the number of QSOs
#          found. Enables an edit button widget if QSOs were found,
#          or disables it if no QSOs were retrieved.
#
# Args:    1. An HList widget, in which the QSOs retrieved will be
#             displayed.
#          2. Instance of an 'edit' button. If the search finds QSOs
#             to be displayed, then his button will be enabled, so
#             that users are able to click on it to edit one of the
#             QSOs.
#          3. Instance of a 'delete' button. See 2.
#          4. Instance of a 'QSO counter' label. The number of QSOs
#             retrieved will be displayed here.
#          5. The SQL string to be used to retrieve the QSOs.
#          6. Reference to the row-ID value of the QSO currently
#             selected.
#          7. Instance of a 'search' button. Used to force a refresh
#             of the log (by executing a new search) after a QSO has
#             been altered and saved.
#          8. Reference to hash containing a copy of configuration
#             file values.
#          9. Reference to array of values that indicate what fields
#             are to be displayed in the view. Typically, this is
#             either array dupeShowFields or logShowFields from the
#             config file.
#          10 Text preceding the 'QSOs found' counter. Typically,
#             this is something like "QSOs found: " :-)
#          11 Reference to array containing instances of 'new QSO'
#             entry fields.
#          12 Instance of a window.
#
# Returns:
#          1. The number of records (QSOs) retrieved by the search.
#          2. The most recent transmitted serial number, or undef
#             if no suitable number was found.
#
######################################################################
#
sub search_and_display {
    my (
        $xList,        $editButton, $deleteButton,
        $counterLabel, $s,          $selectedRowIDRef,
        $searchButton, $hashRef,    $showFieldsRef,
        $labelText,    $nqEntryRef, $window,
    ) = @_;

    my %config       = %{$hashRef};
    my @fieldNames   = @{ $config{fieldNames} };
    my @rightAligned = @{ $config{rightAligned} };
    my @selectedRowID;

    if ( defined @{$selectedRowIDRef} ) {
        @selectedRowID = @{$selectedRowIDRef};
    }

    my $dbh;

    use Tk::ItemStyle;

    # Execute the SQL statement:
    my $ary_ref =
      do_select( $config{db}, $config{tableName}, $config{userName},
        $s, );

    # Process QSOs found:
    my $nTuples          = 0;      # Number of records (QSOs) found.
    my $qsoLine          = "";
    my $foundDupe        = 0;
    my $currentMode      = "";
    my $currentKhz       = "";
    my $currentCall      = "";
    my $mostRecentSerial = undef;

    my $k = 0;
    if ( defined $nqEntryRef ) {
        foreach my $z (@fieldNames) {
            $currentMode = $$nqEntryRef[$k]->get
              if $z eq $config{isMode};
            $currentKhz = $$nqEntryRef[$k]->get
              if $z eq $config{isKhz};
            if ( $z eq $config{isCall} ) {
                $currentCall  = $$nqEntryRef[$k]->get;
                $callEntryBox = $$nqEntryRef[$k];
            }
            $k++;
        }
    }

    if ( defined($ary_ref) ) {
        $nTuples = @{$ary_ref};
        $counterLabel->configure( -text => " $labelText: $nTuples",
        );
        if ( $nTuples > 0 ) {

            # Update the HList widget to display QSOs found:
            $xList->delete('all');
            $xList->configure(
                -command => [
                    \&edit_qso,    $xList, $window,
                    $searchButton, $hashRef,
                ],
            );

            my $bgColour  = $config{hList_bg};
            my $leftStyle = $xList->ItemStyle(
                'text',
                -anchor => 'w',
                -bg     => $bgColour
            );

            my $rightStyle = $xList->ItemStyle(
                'text',
                -anchor => 'e',
                -bg     => $bgColour
            );

            # Process one row at a time:
            my $lastIndex =
              0;    # contains index of last element of an array
            my $rowID =
              0;    # rowID of record, assumed to be at end of array
            my $fieldIndex = 0;
            my $a          = 0;
            foreach my $row ( @{$ary_ref} ) {

                $fieldIndex = 0;
                $lastIndex  = $#{$row};
                $rowID      = ${$row}[$lastIndex];
                $xList->add( $rowID, -data => $rowID );
                $a = 0;
                my $thisMode = "";
                my $thisKhz  = "";
                my $thisCall = "";

                foreach my $field ( @{$row} ) {

                    # Find mode, QRG, call for this QSO
                    # (if we're dupe-checking):
                    if (   defined $field
                        && defined $fieldNames[$fieldIndex] )
                    {
                        if ( $fieldNames[$fieldIndex] eq
                            $config{isMode} )
                        {
                            $thisMode = $field;
                        }
                        if ( $fieldNames[$fieldIndex] eq
                            $config{isKhz} )
                        {
                            $thisKhz = $field;
                        }
                        if ( $fieldNames[$fieldIndex] eq
                            $config{isCall} )
                        {
                            $thisCall = $field;
                        }
                        if ( $fieldNames[$fieldIndex] eq
                            $config{defaultSerialField}
                            && !defined $mostRecentSerial )
                        {
                            $mostRecentSerial = $field
                              if $field =~ /^\d+$/;
                        }
                    }

                  # Align certain fields with the right-hand margin:
                    if ( $rightAligned[$fieldIndex] == 1 ) {
                        if ( ${$showFieldsRef}[$fieldIndex] == 1 ) {
                            $xList->itemCreate(
                                $rowID, $a,
                                -itemtype => 'text',
                                -text     => $field,
                                -style    => $rightStyle
                            );
                            $a++;
                        }
                    }
                    else {

                        # default alignment: left
                        if ( ${$showFieldsRef}[$fieldIndex] == 1 ) {
                            $xList->itemCreate(
                                $rowID, $a,
                                -itemtype => 'text',
                                -text     => $field,
                                -style    => $leftStyle
                            );
                            $a++;
                        }
                    }
                    if (   defined $currentMode
                        && $currentMode eq $thisMode
                        && $thisMode ne "" )
                    {

                        # The modes match...
                        if ( defined $currentKhz && $thisKhz ne "" )
                        {

                      # Check whether the MHz portions are the same.
                            my $a = $currentKhz;
                            my $b = $thisKhz;
                            $a =~ s/\d\d\d$//;
                            $b =~ s/\d\d\d$//;
                            if (   $a eq $b
                                && defined $callEntryBox
                                && $currentCall eq $thisCall
                                && $thisCall ne "" )
                            {

                     # The calls and modes match, MHz is the same,
                     # so it's a dupe. Change background of callsign
                     # entry box to dupe_entry_bg:
                                if ( $rowID > $config{dupeID} ) {
                                    $callEntryBox->configure(
                                        -background =>
                                          $config{dupe_entry_bg} );
                                    $foundDupe = 1;
                                }
                                else {
                                    $callEntryBox->configure(
                                        -background =>
                                          $config{olddupe_entry_bg}
                                      )

               # Don't override dupe_entry_bg with olddupe_entry_bg:
                                      if !$foundDupe;
                                    $foundDupe = 1;
                                }
                            }
                        }
                    }
                    $fieldIndex++;
                }

                if (@selectedRowID) {
                    for my $rid (@selectedRowID) {
                        $xList->selectionSet($rowID)
                          if $rowID eq $rid;
                    }
                }

            }

            $editButton->configure(
                -command => [
                    \&edit_qso,    $xList, $window,
                    $searchButton, $hashRef,
                ],
            ) if defined $editButton;

            if ( defined( $xList->selectionGet ) ) {

           # Turn $editButton and $deleteButton on if QSOs have been
           # selected:
                $editButton->configure( -state => 'normal' )
                  if defined $editButton;
                $deleteButton->configure( -state => 'normal' )
                  if defined $deleteButton;
            }
            else {
                $editButton->configure( -state => 'disabled' )
                  if defined $editButton;
                $deleteButton->configure( -state => 'disabled' )
                  if defined $deleteButton;
            }
        }
        else {

            # print "No QSOs found.\n";
            # ..so none can be selected....turn $editButton off:
            $editButton->configure( -state => 'disabled' )
              if defined $editButton;
            $deleteButton->configure( -state => 'disabled' )
              if defined $deleteButton;

            # Wipe list:
            if ( defined($xList) ) {
                $xList->delete('all');
            }
        }
    }
    else {
        die "No \$ary_ref defined. Runtime error: $!\n";
    }

    # Clear entry box background, only if nu dupes were found...
    $callEntryBox->configure( -background => $config{entry_bg} )
      if defined $callEntryBox && $foundDupe != 1;

    return ( $nTuples, $mostRecentSerial );
}

######################################################################
#
# subroutine do_SQL
#
# Purpose: Deletes or updates a single QSO using '$dbh->do'.
#          (This method can not be used for 'select from' actions
#          because $dbh->do does not return a statement handle.)
#
# Args:    1. The SQL string to be executed.
#          2. Reference to hash containing a copy of configuration
#             file values.
#
# Returns: The $dbh->do return value. Typically '1' for success, or
#          'undef' on failure.
#
######################################################################
#
sub do_SQL {

    my ( $s, $hashRef, ) = @_;

    my %config = %{$hashRef};

    my $dbh;
    eval {
        $dbh = DBI->connect(
            "dbi:SQLite:dbname=$config{db}",
            "", "",
            {
                RaiseError => 0,
                PrintError => 1,
                AutoCommit => 1,
            }
        );
    };

    # DBI doesn't seem to flag *any* errors at this point
    # Pointless error-trapping code follows:

    if ($@) {
        print "\n";
        print "Error: Could not connect to database $config{db}\n";
        print "Database driver returned: $dbh->errstr \n"
          if defined($dbh);
        print "DBI\:\:connect returned undef for \$\$dbh.\n";
        die;
    }

    if ( !defined($dbh) ) {
        print "\n";
        print "Error: Could not connect to database $config{db}\n";
        print "DBI\:\:connect returned undef for \$\$dbh.\n";
        die;
    }

    if ( !$dbh ) {
        print "\n";
        print "Error: Could not connect to database $config{db}\n";
        print "DBI\:\:connect:  returned 'false'.\n";
        print "DBI::errstr\n";
        die;
    }

    # print "\nSQL statement: $s\n";
    my $rv = $dbh->do("$s");

    # Trap failures:
    if ($@) {
        print "\n";
        print "Error updating record in database $config{db}\n";
        print "SQL statement: $s\n";
        print "Database driver returned: $dbh->errstr\n";
        print "DBI             says    : $DBI::errstr\n";
        die;
    }

    if ( not defined($rv) || $rv != 1 ) {
        print "\n";
        print "Error trying to update or delete the QSO:\n";
        print "SQL statement: $s\n";
        print "return value : $rv\n";
        die;
    }
    $dbh->disconnect;
    return ($rv);
}

######################################################################
#
# subroutine config_cwd:
#
# Purpose: Open socket, set WPM, weight, tone and volume on CW-daemon.
#
# Args:    Reference to hash containing a copy of configuration file
#          values.
#
# Returns: 1. Integer indicating that CW-daemon is not usable (if '1')
#          2. The Internet socket that has been opened to the CW
#             daemon.
#
######################################################################
#
sub config_cwd {

# $junk catches unwanted args when used as a callback, and disregards
# it.
    my ( $hashRef, $junk ) = @_;
    %config = %{$hashRef};

    if ( $config{noCwDaemon} ) {
        print
          "\nCW support disabled in config file: noCwDaemon = 1 \n"
          if $config{debug};
        return ( 1, 0 );
    }

    my $found = 0;
    my @processList;

    @processList = `ps ax` unless $OSNAME =~ /win/i;

    if ( !@processList ) {
        if ( $config{debug} ) {
            print
              "\nOperating system does not provide process list.\n";
            print
"Can't determine PID of cwdaemon. CW support disabled.\n";
        }
    }
    else {
        for my $ps (@processList) {
            $found = 1 if $ps =~ /cwdaemon/;    # Found cwdaemon.
        }
    }

    if ( !$found ) {
        $config{noCwDaemon} = 1;
        print
"\nCould not find cwdaemon process. CW support disabled.\n"
          if $config{debug};
        ${$hashRef}{noCwDaemon} = 1;
        return ( 1, 0 );
    }

    $cwdEsc          = "\033";
    $cwdSpeedPrefix  = $config{cwdSpeedPrefix};
    $cwdTonePrefix   = $config{cwdTonePrefix};
    $cwdWeightPrefix = $config{cwdWeightPrefix};
    $cwdVolumePrefix = $config{cwdVolumePrefix};
    $cwdPttPrefix    = $config{cwdPttPrefix};
    $cwdAbortPrefix  = $config{cwdAbortPrefix};

    $cwdWpm      = $config{cwdWpm};
    $cwdWeight   = $config{cwdWeight};
    $cwdSideTone = $config{cwdSideTone};
    $cwdVolume   = $config{cwdVolume};

    $cwdHost = $config{cwdHost};
    $cwdPort = $config{cwdPort};

    use IO::Socket::INET;

    $cwdSocket = IO::Socket::INET->new(
        PeerAddr => "$cwdHost",
        PeerPort => "$cwdPort",
        Proto    => "udp",
        Type     => SOCK_DGRAM,
        Blocking => 0,
      )
      or do {

        print "Unable to open cwd socket $cwdHost:$cwdPort\n";

        # Quit subroutine; indicating CWdaemon not usable:
        return ( 1, 0 );
      };

    # First, abort any CW being sent, to avoid any funny business
    # as we tweak the daemon:
    $cmd = $cwdEsc . $cwdAbortPrefix;
    return ( 1, 0 )
      if send_to_cwd( $cwdSocket, $cmd, $config{debug} ) == 1;

    # PTT off:
    $cmd = $cwdEsc . $cwdPttPrefix . "0";
    return ( 1, 0 )
      if send_to_cwd( $cwdSocket, $cmd, $config{debug} ) == 1;

    # Sound off:
    $cmd = $cwdEsc . $cwdTonePrefix . "0";
    return ( 1, 0 )
      if send_to_cwd( $cwdSocket, $cmd, $config{debug} ) == 1;

    my $cmd = $cwdEsc . $cwdSpeedPrefix . $cwdWpm;
    return ( 1, 0 )
      if send_to_cwd( $cwdSocket, $cmd, $config{debug} ) == 1;

    $cmd = $cwdEsc . $cwdWeightPrefix . $cwdWeight;
    return ( 1, 0 )
      if send_to_cwd( $cwdSocket, $cmd, $config{debug} ) == 1;

    # PTT on:
    $cmd = $cwdEsc . $cwdPttPrefix . "1";
    return ( 1, 0 )
      if send_to_cwd( $cwdSocket, $cmd, $config{debug} ) == 1;

    # Sound on:
    $cmd = $cwdEsc . $cwdTonePrefix . $cwdSideTone;
    return ( 1, 0 )
      if send_to_cwd( $cwdSocket, $cmd, $config{debug} ) == 1;

    # Must be last, or else it won't work:
    $cmd = $cwdEsc . $cwdVolumePrefix . $cwdVolume;
    return ( 1, 0 )
      if send_to_cwd( $cwdSocket, $cmd, $config{debug} ) == 1;

    return ( 0, $cwdSocket );
}

######################################################################
#
# subroutine config_rigctld:
#
# Purpose: Open socket to rigctld on UDP port 4532
#
# Args:    1. Reference to hash containing a copy of configuration
#             file values.
#          2. Reference to window.
#          3. Reference to scalar that locks out the polling of the
#             rig.
#
# Returns: 1. The opened rigctld UDP socket
#          2. Variable which, if ==1, indictates the rig is not
#             available.
#          3. The mode
#          4. The frequency in kHz
#
######################################################################
#
sub config_rigctld {

    my ( $hashRef, $window, $rigLockRef ) = @_;

    $$rigLockRef = 1;

    %config = %{$hashRef};
    my @fieldTypes = @{ $config{fieldTypes} };

    my $rigTypesFound = 0;
    foreach my $k (@fieldTypes) {
        $rigTypesFound = 1 if $k eq "RIG";
    }

    # Quit if useRig not '1':
    if ( $config{useRig} == 0 ) {
        if ($rigTypesFound) {
            my $w1 = "Warning:\n config file states: 'useRig=0'\n";
            my $w2 = "so rig support is turned off.\n\n";
            my $w3 = "\n";
            my $w4 = "Field type 'RIG' is not available.";
            my_dialog( $hashRef, $window, "Warning", "Close",
                $config{dialogGeometry}, "$w1$w2$w3$w4" );
        }

        change_rigtype($hashRef);
        $$rigLockRef = 0;
        return ( undef, 1, 0, 0 );
    }

    # OK. The user wants rig support.....

    my $found = 0;
    my @processList;

    @processList = `ps ax` unless $OSNAME =~ /win/i;
    if ( !@processList ) {
        if ( $config{debug} ) {
            print
              "\nOperating system does not provide process list.\n";
            print
"Can't determine PID of rigctld. Rig support disabled.\n";
        }
    }
    else {
        for my $ps (@processList) {
            $found = 1 if $ps =~ /rigctld/;    # Found rigctld
        }
    }

    if ( !$found ) {
        if ( $config{debug} ) {
            print "\nCould not find rigctld process. ";
            print "Rig support disabled: 'useRig=0'\n";
        }
        ${$hashRef}{useRig} = 0;

        if ($rigTypesFound) {
            my $w1 =
"Rig supported requested in config file by 'useRig'\n";
            my $w2 = "However, no \'rigctld\' process was found,\n";
            my $w3 = "so Rig support is turned off\n";
            my $w4 = "and field type 'RIG' is not available.";
            my_dialog( $hashRef, $window, "Warning", "Close",
                $config{dialogGeometry}, "$w1$w2$w3$w4" );
        }

        change_rigtype($hashRef);
        $$rigLockRef = 0;
        return ( undef, 1, 0, 0 );
    }

    my $rigctldPort = $config{rigctldPort};
    my $rigctldHost = $config{rigctldHost};

    use IO::Socket::INET;
    my $deadSocket    = 0;
    my $rigctldSocket = IO::Socket::INET->new(
        PeerAddr => "$rigctldHost",
        PeerPort => "$rigctldPort",
        Proto    => "tcp",
        Blocking => 0,
    ) or $deadSocket = 1;

    if ( $deadSocket != 0 ) {
        print "\'rigctld\' is in the proccess list but I can not\n";
        print "open rigctld socket $rigctldHost:$rigctldPort\n";

        # Quit subroutine; indicating rigctld not usable:

        my $w1 =
"Rig supported has been requested in config file by 'useRig'\n";
        my $w2 =
          "A \'rigctld\' process was found, but the socket could\n";
        my $w3 = "not be opened. Rig support is turned off\n";
        my $w4 = "and field type 'RIG' is not available.";
        my_dialog( $hashRef, $window, "Warning", "Close",
            $config{dialogGeometry}, "$w1$w2$w3$w4" );
        change_rigtype($hashRef);
        $$rigLockRef = 0;
        return ( undef, 1, 0, 0 );
    }

    print $rigctldSocket "\\get_mode\n";
    print $rigctldSocket "\\get_freq\n";
    my ( $freq, $mode, $err ) =
      get_rig_data( $rigctldSocket, $config{debug} );

    print "mode=$mode\n" if $config{debug};
    print "kHz=$freq\n"  if $config{debug};

    $$rigLockRef = 0;
    return ( $rigctldSocket, 0, $mode, $freq );
}

######################################################################
#
# subroutine send_to_cwd
#
# Purpose: Send string to cwdaemon on Internet socket.
#
# Args:    1. Socket on which to transmit string
#          2. String to be sent.
#          3. '1' if in debug mode. Else 0..
#
# Returns: 1 for failure; 0 for success.
#
######################################################################
#

sub send_to_cwd {

    my ( $cwdSocket, $cmd, $debug ) = @_;
    print "sending $cmd\n" if $debug;
    if ( !print $cwdSocket $cmd ) {
        print
"\nCould not print to cwdaemon socket; command=\"$cmd\".\n"
          if $debug;
        return (1);
    }

    return (0);
}

######################################################################
#
# subroutine my_dialog
#
# Purpose: Replaces Tk::Dialog which leaves a blocked keyboard on
#          IceWM.
#          This subroutine pops up a dialog box which has a title, a
#          single button, a certain geometry and contains one or more
#          text labels. The dialog is a toplevel 'always on top' and
#          performs a local grab for the keyboard. The user reads the
#          text labels, and closes the dialog by pushing the button.
#
# Args:    1. Reference to hash holding copies of config file values.
#             Used to retrieve window types such as colour, relief
#             etc.
#          2. Instance of window in which to pop up the dialog.
#          3. Text string that is the title of the dialog window.
#          4. Array of strings. Each string is displayed in a
#             pushbutton.
#          5. Text string that represents the geometry to be used.
#          6. Array of strings. Each string becomes a text label in
#             the dialog box.
#
# Returns: Instance of the dialog window.
#
######################################################################
#
sub my_dialog {

    my (
        $hashRef,    $window,   $title,
        $buttonText, $geoMetry, @labelArray
    ) = @_;

    my %config = %{$hashRef};

    my $dg = $config{dialogGeometry};
    $dg =~ /^(\d+)x/;
    my $xWidth     = $1;
    my $wrapLength = 10;
    $wrapLength = $xWidth - 10 if $xWidth >= 10;

    my $dialogWindow = $window->Toplevel(
        -title       => $title,
        -relief      => 'raised',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    );
    $dialogWindow->geometry($geoMetry);

    foreach my $a (@labelArray) {
        $dialogWindow->Label(
            -text        => $a,
            -borderwidth => $config{borderwidth},
            -foreground  => $config{fg},
            -background  => $config{bg},
            -wraplength  => "$wrapLength",
        )->pack( -expand => 1, -fill => 'both', -side => 'top', );
    }

    my $okButton = $dialogWindow->Button(
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -text             => $buttonText,
        -underline        => 0,
        -command          => sub {
            $dialogWindow->grabRelease;
            $dialogWindow->destroy;
        },
      )->pack(
        -expand => 1,
        -fill   => 'x',
        -side   => 'bottom',
        -anchor => 's',
      );

    my @p        = split( / */, $buttonText );
    my $keyUpper = "<Alt-" . uc( $p[0] ) . ">";
    my $keyLower = "<Alt-" . lc( $p[0] ) . ">";
    $dialogWindow->bind(
        "$keyUpper" => sub {
            $okButton->flash;
            $okButton->invoke;
        }
    );

    $dialogWindow->bind(
        "$keyLower" => sub {
            $okButton->flash;
            $okButton->invoke;
        }
    );

    # Make dialog prominently visible
    # There seems to be no way to push a child widget above another
    # toplevel.. ...unless the parent is above those toplevels.
    $window->raise;    # so raise parent first...
                       # $dialogWindow->focusmodel('active');
    $dialogWindow->transient( $dialogWindow->Parent->toplevel );
    $dialogWindow->raise;    # Now make child visible
    $okButton->focus;

    #$dialogWindow->waitWindow;
    # Return $dialogWindow so the main code can keep raising it,
    # if required.
    return ($dialogWindow);
}

######################################################################
#
# subroutine exit_program
#
# Purpose: Replaces Tk::Dialog which leaves a blocked keyboard on IceWM.
#          This subroutine pops up a dialog box and offers to exit the
#          program. There are two buttons: "OK" and "Cancel".
#          The dialog is a toplevel 'always on top' and performs a
#          local grab for the keyboard. The user closes the dialog by
#          by pushing one of the buttons.
#
# Args:    1. Reference to hash holding copies of config file values.
#             Used to retrieve window types such as colour, relief etc.
#          2. Instance of window in which to pop up the dialog.
#
# Returns: Nothing. This is a call-back.
#
######################################################################
#
sub exit_program {
    my ( $hashRef, $window ) = @_;

    my %config = %{$hashRef};

    my $dg = $config{dialogGeometry};
    $dg =~ /^(\d+)x/;
    my $xWidth     = $1;
    my $wrapLength = 10;
    $wrapLength = $xWidth - 10 if $xWidth >= 10;

    my $dialogWindow = $window->Toplevel(
        -title       => " Exit dialog ",
        -relief      => 'raised',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    );
    $dialogWindow->geometry( $config{dialogGeometry} );

    my $continueButton = $dialogWindow->Button(
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -underline        => '2',
        -text             => " Continue program ",
        -command          => sub {
            $dialogWindow->grabRelease;
            $dialogWindow->destroy;
        },
      )->pack(
        -expand => 1,
        -fill   => 'x',
        -side   => 'top',
        -anchor => 'n',
      );
    $dialogWindow->bind(
        '<Alt-O>' => sub {
            $continueButton->invoke();
        }
    );

    $dialogWindow->bind(
        '<Alt-o>' => sub {
            $continueButton->invoke();
        }
    );

    $dialogWindow->bind(
        '<Escape>' => sub {
            $continueButton->invoke();
        }
    );

    $dialogWindow->Label(
        -text        => " Exit program? ",
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
        -wraplength  => "$wrapLength",
    )->pack( -expand => 1, -fill => 'both', -side => 'top', );

    my $exitButton = $dialogWindow->Button(
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -underline        => '2',
        -text             => " Exit program ",
        -command          => sub {
            $dialogWindow->grabRelease;
            $dialogWindow->destroy;
            send_msge( undef, '\<', undef, $hashRef, $cwdSocket,
                undef, undef )
              if $config{qrvSk} == 1;
            $window->destroy;
            exit;
        },
      )->pack(
        -expand => 1,
        -fill   => 'x',
        -side   => 'bottom',
        -anchor => 's',
      );
    $dialogWindow->bind(
        '<Alt-X>' => sub {
            $exitButton->invoke();
        }
    );
    $dialogWindow->bind(
        '<Alt-x>' => sub {
            $exitButton->invoke();
        }
    );

    # Always on top:
    $dialogWindow->transient( $dialogWindow->Parent->toplevel );

    # $dialogWindow->grab;
    $dialogWindow->raise;
    $exitButton->focus;
}

######################################################################
#
# subroutine show_copyright
#
# Purpose: Replaces Tk::Dialog which leaves a blocked keyboard on
#          IceWM. This subroutine shows the copyright, and offers to
#          continue or to exit the program.
#          There are two buttons: "OK" and "Cancel".
#          The dialog is a toplevel 'always on top' and performs a
#          local grab for the keyboard. The user closes the dialog by
#          by pushing one of the buttons.
#
# Args:    1. Reference to hash holding copies of config file values.
#             Used to retrieve window types such as colour, relief
#             etc.
#          2. Instance of window in which to pop up the dialog.
#          3. Copyright text to be displayed.
#
# Returns: Nothing. This is a call-back.
#
######################################################################
#
sub show_copyright {
    my ( $hashRef, $window, $copyRight ) = @_;

    my %config = %{$hashRef};

    my $dialogWindow = $window->Toplevel(
        -title       => " Copyright dialog ",
        -relief      => 'raised',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    );
    $dialogWindow->geometry( $config{copyrightGeometry} );

    my $exitButton = $dialogWindow->Button(
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -underline        => '2',
        -text             => " Exit program ",
        -command          => sub {
            $dialogWindow->grabRelease;
            $dialogWindow->destroy;
            exit;
        },
      )->pack(
        -expand => 1,
        -fill   => 'x',
        -side   => 'top',
        -anchor => 'n',
      );
    $dialogWindow->bind(
        '<Alt-X>' => sub {
            $exitButton->invoke();
        }
    );
    $dialogWindow->bind(
        '<Alt-x>' => sub {
            $exitButton->invoke();
        }
    );

    use Tk::Bitmap;

    #   use Tk::Image;
    my $kiwi = "kiwi-blk-52x52.xbm";
    my $image = $window->Bitmap( -file => $kiwi )
      or warn "Image file $kiwi not found :-( \n";

    $dialogWindow->Label(
        -text       => "",
        -foreground => $config{fg},
        -background => $config{bg},
    )->pack;
    $dialogWindow->Label( -image => $image, -relief => 'sunk', )
      ->pack;

    $dialogWindow->Label(
        -text        => $copyRight,
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    )->pack( -expand => 1, -fill => 'both', -side => 'top', );

    my $continueButton = $dialogWindow->Button(
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -underline        => '2',
        -text             => " Continue ",
        -command          => sub {
            $dialogWindow->grabRelease;
            $dialogWindow->destroy;
        },
      )->pack(
        -expand => 1,
        -fill   => 'x',
        -side   => 'bottom',
        -anchor => 's',
      );

    $continueButton->bind(
        '<Alt-O>' => sub {
            $continueButton->invoke();
        }
    );

    $continueButton->bind(
        '<Alt-o>' => sub {
            $continueButton->invoke();
        }
    );
    $continueButton->bind(
        '<Escape>' => sub {
            $continueButton->invoke();
        }
    );

    # Always on top:
    $dialogWindow->transient( $dialogWindow->Parent->toplevel );

    #    $dialogWindow->grab;
    $dialogWindow->raise;
    $continueButton->focus;
    return ($dialogWindow);
}

######################################################################
#
# subroutine set_focus
#
# Purpose: Puts focus on entry box indicated by 'defaultEntryField'
#          in the config file (typically: the 'call' field).
#
# Args:    1. Reference to hash holding copies of config file values.
#             Used to retrieve window types such as colour, relief
#             etc.
#          2. Reference to nqEntry array, which contains instances of
#             all the entry boxes in the 'new QSO template' frame.
#
# Returns: Void.
#
######################################################################
#
sub set_focus {

    my ( $hashRef, $nqEntryRef ) = @_;

    my %config     = %{$hashRef};
    my @fieldNames = @{ $config{fieldNames} };

    # Focus on defaultEntryField
    #
    for ( my $i = 0 ; $i <= $#fieldNames ; $i++ ) {
        ${$nqEntryRef}[$i]->focus
          if $fieldNames[$i] eq $config{defaultEntryField};
    }
}

######################################################################
#
# subroutine cw_kbd
#
# Purpose: Opens a CW keyboard in a new window.
#
#          The window contains an entry box and a 'Close' button.
#          Anything typed into the entry box is validated by
#          subroutine val_cwkbd which passes the character on to
#          cwdaemon for transmission as CW.
#
#          Pushing the 'Close' button or typing Alt-[k|K] destroys the
#          window.
#
# Returns: Void.
#
######################################################################
#
sub cw_kbd {
    my ( $window, $hashRef, $cwWindowRef, $cwdSocket ) = @_;

    my %config = %{$hashRef};

    # Pop up the editor and put a frame in it:
    ${$cwWindowRef} = $window->Toplevel(
        -title       => " CW keyboard ",
        -relief      => 'raised',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    );

    my $cwWindow = ${$cwWindowRef};
    $cwWindow->geometry( $config{cwkbdGeometry} );

    $cwWindow->Label(
        -anchor     => 'center',
        -background => $config{labels_bg},
        -foreground => $config{labels_fg},
        -takefocus  => 0,
        -text => " * AR   = BT   < SK   ( KN   ! SN   & AS   > BK "
    )->pack;

    my $cwEntry = $cwWindow->Entry(
        -validate => 'key',
        -validatecommand =>
          [ \&val_cwkbd, $hashRef, $cwdSocket, \@_, ],
      )->pack(
        -expand => 1,
        -fill   => 'x',
      );
    my $closeButton = $cwWindow->Button->pack(
        -fill   => 'x',
        -expand => 1,
        -side   => 'left',
        -anchor => 's',
    );

    $closeButton->configure(
        -activeforeground => $config{buttons_activefg},
        -activebackground => $config{buttons_activebg},
        -foreground       => $config{buttons_fg},
        -background       => $config{buttons_bg},
        -underline        => '0',
        -text             => "Close",
        -command          => sub {
            $cwWindow->destroy;
            $$cwWindowRef = undef;
        }
    );

    $cwWindow->bind(
        '<Alt-c>' => sub {
            $closeButton->invoke;
        }
    );
    $cwWindow->bind(
        '<Alt-C>' => sub {
            $closeButton->invoke;
        }
    );
    $cwWindow->bind(
        '<Escape>' => sub {
            $closeButton->invoke;
        }
    );

    $cwWindow->raise;
    $cwEntry->focus;

}

######################################################################
#
# subroutine val_cwkbd
#
# Purpose: checks each character entered into the CW keyboard entry
#          widget as it is being typed. Illegal characters are
#          ignored. Valid characters are escaped if necessary before
#          being sent to the cwdaemon.
#
# Args:
#
# 1. Reference to config hash.
# 2. Already opened internet UDP socket to cwdaemon.
# 3. Junk, unused callback cruft.
# 4. The proposed new string in the entry widget.
# 5. The latest character to be added to the entry widget.
# 6. The existing (soon to be old) string in $entry.
# 7. The index number of the character just entered into the entry
#     widget.
# 8. A number indicating Insert Mode (1) or Delete Mode (0).
#
# Returns: 1 (true)  if valid. The character appears in the entry.
#          0 (false) if not valid. The character is ignored.
#
######################################################################
#
sub val_cwkbd {

    my ( $hashRef, $cwdSocket, $junk, $newString, $newCharacter,
        $oldString, $charIndex, $insertMode, )
      = @_;

    # Do nothing if called for nothing:
    return (0) if !defined($newCharacter);

    # Perform delete (but no CW) in case of a delete action:
    return (1) if $insertMode == 0;

    # Don't allow, unless character is legal:
    return (0)
      unless $newCharacter =~ /[A-Za-z0-9\/\*\=\<\(\!\&\>\? ]/;

    if ( length($newString) > 0 ) {

        # Send as CW:
        if ( $newCharacter =~ /[\/\*\=\<\(\!\&\>\? ]/ ) {

            # Escape from funny business:
            $newCharacter = '\\' . "$newCharacter";
        }

        send_msge( undef, $newCharacter, undef, $hashRef,
            $cwdSocket, undef, undef );
    }
    else {

        # Do nothing on empty string:
        return (0);
    }

    # Catch-all: OK to echo it.
    return (1);
}

######################################################################
#
# callback clear-template
#
# Purpose: this callback is executed when the user hits 'Alt-w' to
#          wipe the new QSO template.
#
# Args: 1. A reference to an array containing default field
#          values. This is array 'defaultValues' in the config file.
#
#       2. A reference to an array containing entry field values (the
#          stuff that is in the field entry boxes).
#
#       3. A reference to an array containing the type of the entry
#          fields (things like 'RIG', '---', 'LCK' etc etc.
#
# Returns: void
#
######################################################################

sub clear_template {

    my ( $defaultValuesRef, $nqEntryRef, $fieldTypesRef, $deField )
      = @_;

    # Clear entries according to  field status

    for ( my $i = 0 ; $i <= $#{$nqEntryRef} ; $i++ ) {

        # Keep contents of all fields except '---' fields:
        # UTC and RIG fields are automatically refreshed by
        #  sub update_entries.
        if ( ${$fieldTypesRef}[$i] eq "---" ) {

            # Wipe '---' fields:
            my $l = length( ${$nqEntryRef}[$i]->get() );
            ${$nqEntryRef}[$i]->delete( 0, $l );
        }

    }
    $$nqEntryRef[$deField]->focus();
}

#####################################################################
#
# subroutine init_serialfield
#
# Purpose: this subroutine is executed when a type change takes place
# on the serial number field, or when the serial field needs to be
# initialised as part of program start-up.
#
# Args: 1. The actual value of in the field entry box. Usually a
#          pre-defined starting number from the config file, or
#          possibly an empty string.
#
#       2. The the default value for the serial number as defined in
#          the config file.
#
#       3. The type of the field. One of "CAR", "LCK", "---" or "+1".
#       4. Reference to scalar holding the most recently transmitted
#          serial number
#       5. Debug flag: '1' to get debug messages to the console.
#
# Returns: void
#
#####################################################################

sub init_serialfield {

    my ( $nqEntry, $defaultValue, $fieldType, $mostRecentSerialRef,
        $debug )
      = @_;

    # OK. This is a serial field, and possibly of type "+1".
    # Come up with a sensible starting value,
    # and disable access to the entry field.
    my $oldVal = $nqEntry->get();
    my $num    = $oldVal;
    if ($debug) {
        print "Found in log: most-recent transmitted serial:"
          . "$$mostRecentSerialRef\n"
          if defined $$mostRecentSerialRef;
    }
    if ( $oldVal !~ /^\d+/ ) {

        if ( defined $$mostRecentSerialRef ) {
            $num = $$mostRecentSerialRef;
            $num++ if $$mostRecentSerialRef =~ /^\d+$/;
        }
        else {

            # There is no useful starting number in the box..
            if ( $defaultValue =~ /^\d+/ ) {

                # ..so we'll use the default number from the config
                #file:
                $num = $defaultValue;
            }
            else {

                # No useful starting serial anywhere..so use '001':
                $num = "001";
            }
        }
    }

    $nqEntry->configure( -state => 'normal' );
    my $l = length( $nqEntry->get() );
    $nqEntry->delete( 0, $l );

    $nqEntry->insert( 0, $num )
      if $fieldType eq "+1";

    $nqEntry->insert( 0, $defaultValue )
      if $fieldType eq "---" && $defaultValue ne "";

    $nqEntry->configure( -state => 'disabled' )
      if $fieldType eq "+1" || $fieldType eq "LCK";

}

#####################################################################
#
# callback utc_clock
#
# Purpose: This code is executed when the 'Use UTC clock' button is
#          released. It is also invoked at program start-up. If the
#          user wants automatic UTC in clock fields, then the
#          callback disables those Entry fields for user access and
#          sets the type to UTC. If the user has chosen not to use
#          the UTC clock, then those fields that are clock fields
#          become accessible to the user, and the field type is set
#          according to $config{fieldTypes[$i]. Fields that are not
#          $config{clockData} remain untouched.
#
# Args:
#
#       1. Reference to config file hash.
#       2. Reference to array containing state of option menus
#       3. Reference to value of 'Use UTC clock' checkbutton.
#       4. Reference to array containing instances of option menus.
#       5. Reference to array containing instances of field entry
#          boxes.
#
# Returns: nothing, not even undef, as this is a callback.
#
#####################################################################

sub utc_clock {

    my (
        $hashRef,    $fieldTypesRef, $valueRef,
        $typeOmsRef, $nqEntryRef
    ) = @_;

    # Make local copy of config hash:
    my %config = %{$hashRef};

    for ( my $i = 0 ; $i < $#$fieldTypesRef ; $i++ ) {

        if ( $config{clockData}[$i] == 1 ) {

            # We're dealing with a clock field.

            if ( $$valueRef == 1 ) {

                # Checkbutton is '1'. Force UTC type on field:
                $$fieldTypesRef[$i] = "UTC";

                # 'type' option menu has been forced. Now disable
                # access:
                $$typeOmsRef[$i]->configure( -state => 'disabled' );

                # Also disable user access to UTC field entry box:
                $$nqEntryRef[$i]->configure( -state => 'disabled' );
            }
            else {

             # This field contains clock data, but the user does not
             # want automatic UTC (checkbutton is '0'):
             # Set type as per fieldTypes in config file:

                $$fieldTypesRef[$i] = $config{fieldTypes}[$i];

                # Allow user access to option menu:
                $$typeOmsRef[$i]->configure( -state => 'normal' );

                # Allow user access to entry box if 'CAR' or '---':
                if ( $$fieldTypesRef[$i] =~ /CAR|---/ ) {
                    $$nqEntryRef[$i]
                      ->configure( -state => 'normal' );
                }
                else {
                    $$nqEntryRef[$i]
                      ->configure( -state => 'disabled' );
                }
            }
        }
        else {

            # Field does not contain clock data.
            # Do nothing.
        }
    }
}

######################################################################
#
# subroutine stat_frames
#
# Purpose: Frames a status line, which is partitioned up into segments
#
# Args: The configuration hash
#       Instance of a frame, in which to set up the segmented status
#       frame
#
# Returns: The four segments of the status frames, as a flat list,
#          left to right. Also returns an additional DXCC status
#          line, unpartitioned.
#
######################################################################

sub stat_frames {

    my ( $configRef, $swFrame ) = @_;

    my %config = %{$configRef};

    # Set up status line(s), below hList:
    my $dxccFrame = $swFrame->Frame(
        -relief      => 'flat',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    )->pack( -expand => 0, -fill => 'x', -anchor => 's' );

    my $statFrame = $swFrame->Frame(
        -relief      => 'flat',
        -borderwidth => $config{borderwidth},
        -foreground  => $config{fg},
        -background  => $config{bg},
    )->pack( -expand => 0, -fill => 'x', -anchor => 's' );

    my $statFrame_2 = $swFrame->Frame(
        -relief      => 'sunken',
        -foreground  => $config{fg},
        -background  => $config{bg},
        -borderwidth => $config{borderwidth},
      )->pack(
        -expand => 1,
        -fill   => 'x',
        -anchor => 's',
        -side   => 'bottom'
      );

    my $statFrame_l = $statFrame->Frame(
        -foreground => $config{fg},
        -background => $config{bg},
      )->pack(
        -expand => 1,
        -fill   => 'x',
        -side   => 'left',
      );

    my $statFrame_ml = $statFrame->Frame(
        -foreground => $config{fg},
        -background => $config{bg},
      )->pack(
        -expand => 1,
        -fill   => 'x',
        -side   => 'left',
      );

    my $statFrame_mr = $statFrame->Frame()->pack(
        -expand => 1,
        -fill   => 'x',
        -side   => 'left',
    );

    my $statFrame_r = $statFrame->Frame()->pack(
        -expand => 1,
        -fill   => 'x',
        -side   => 'left',
    );
    return (
        $statFrame_l, $statFrame_ml, $statFrame_mr,
        $statFrame_r, $dxccFrame
    );
}

######################################################################
#
# subroutine reverse_array
#
# Purpose: Reverses an array
#
# Args: Reference to array
#
# Returns: void.
#
######################################################################

sub reverse_array {

    my ($arrayRef) = @_;

    # Reverse array @rowIDs so the ADIF file is in
    # chronological order:
    my @tempArray;
    my $index = 0;
    do {
        $tempArray[$index] = pop( @{$arrayRef} );
        $index++;
    } while $#{$arrayRef} >= 0;
    @{$arrayRef} = @tempArray;
}

######################################################################
#
# subroutine process_cty_dat
#
# Purpose: Reads file $config{ctyDat}, turning the info into hashes.
#          There are country, continent, CQ zone and ITU zone hashes,
#          keyed to a prefix.
#
# Args:
#          Reference to country hash
#          Reference to continent hash
#          Reference to CQ zone hash
#          Reference to ITU zone hash
#          Name of cty.dat file
#
# Returns: void.
#
######################################################################

sub process_cty_dat {

    my (
        $ctyHashRef,   $contHashRef, $cqzHashRef,
        $ituzHashRef, $ctyDat
    ) = @_;

    my $ctyDatFH;
    if ( !( open( $ctyDatFH, '<', $ctyDat ) ) ) {
        print "Could not open file $ctyDat\n";
        return ();
    }

    my $cty   = "";
    my $cont  = "";
    my $cqz   = "";
    my $ituz = "";
    my $lat   = "";
    my $long  = "";
    my $ut    = "";
    my $pfx   = "";

    while (<$ctyDatFH>) {
        chomp;
        if ( $_ =~ m/^[A-Z]{1}/ ) {
            $cty   = "";
            $cont  = "";
            $cqz   = "";
            $ituz = "";
            $lat   = "";
            $long  = "";
            $ut    = "";
            $pfx   = "";

            # String starts with capital: is 1st line:
            ( $cty, $cqz, $ituz, $cont, $lat, $long, $ut, $pfx ) =
              split( /:/, $_ );

            # Strip leading spaces and other junk:
            $cqz   =~ s/^\s+//;
            $ituz =~ s/^\s+//;
            $cont  =~ s/^\s+//;
            $lat   =~ s/^\s+//;
            $long  =~ s/^\s+//;
            $ut    =~ s/^\s+//;
            $pfx   =~ s/^\s+//;
            $pfx   =~ s/;//;
            $$ctyHashRef{$pfx}   = $cty;
            $$contHashRef{$pfx}  = $cont;
            $$cqzHashRef{$pfx}   = $cqz;
            $$ituzHashRef{$pfx} = $ituz;
        }
        else {

            # String contains additional prefixes:
            foreach my $extrapfx ( split( /,/, $_ ) ) {
                $extrapfx =~ s/ *//;
                $extrapfx =~ s/;\s+//;
                my $p = $extrapfx;
                $extrapfx =~ s/\(\d{1,2}\)//;
                $extrapfx =~ s/\[\d{1,2}\]//;
                $$ctyHashRef{$extrapfx}  = $cty;
                $$contHashRef{$extrapfx} = $cont;
                if ( $p =~ /\((\d{1,2})\)/ ) {
                    $$cqzHashRef{$extrapfx} = $1;
                }
                else {
                    $$cqzHashRef{$extrapfx} = $cqz;
                }
                if ( $p =~ /\[(\d{1,2})\]/ ) {
                    $$ituzHashRef{$extrapfx} = $1;
                }
                else {
                    $$ituzHashRef{$extrapfx} = $ituz;
                }
            }
        }
    }
}    # end of sub process_cty_dat

######################################################################
#
# subroutine process_scp
#
# Purpose: Reads file $config{masterScp}, turning the info into an
#          array.
#
# Args:
#          Reference to super check partial array
#          Name of super chack partial file
#
# Returns:
#          '1' if processing of super check partial file was
#           successful.
#
######################################################################

sub process_scp {

    my ( $scpArrayRef, $masterScp ) = @_;

    my $masterScpFH;
    if ( !( open( $masterScpFH, '<', $masterScp ) ) ) {
        print "Could not open file $masterScp\n";
        return (0);
    }
    while (<$masterScpFH>) {
        chomp;
        push @$scpArrayRef, $_;
    }
    return (1);
}    # end of sub process_scp

######################################################################
#
# subroutine update_scp
#
# Purpose: Updates super check partial info, provided the callsign
#          field holds a call.
#
# Args:
#
#   1. Reference to array of 'new QSO' entry box widgets. These are
#      the entry fields of the new QSO template.
#   2. Reference of hash containing the config values.
#   3. Reference to array containing the names of the fields, as per
#      config file.
#   4. Reference to -labelvariable, used to display SCP string
#
# Returns: void.
#
######################################################################
#
sub update_scp {

    my (
        $nqEntryRef,  $hashRef, $fieldNamesRef,
        $scpArrayRef, $scpStringRef
    ) = @_;

    my %config     = %{$hashRef};
    my @nqEntry    = @{$nqEntryRef};
    my @fieldNames = @{$fieldNamesRef};

    print "Executing SCP\n" if $config{debug};

    # Scan array @fieldNames to find callsign field:
    for ( my $i = 0 ; $i <= $#fieldNames ; $i++ ) {

        if ( $fieldNames[$i] =~ /($config{isCall})/ ) {

            # This field is the callsign field:

            my $scpString = "";
            eval { defined $nqEntry[$i]->get } or return;
            my $callSign = $nqEntry[$i]->get;

            if ( $callSign ne "" ) {

                # Check to see if callsign produces a direct match:
                foreach my $x (@$scpArrayRef) {
                    $scpString = $scpString . "$x\n"
                      if $x =~ m/^$callSign/;
                }
            }

            # Display $scpString:
            $$scpStringRef = $scpString;
        }
    }
}    # End of sub update_scp

######################################################################

=head1 NAME

  qle - a general-purpose QSO logger and log editor

=head1 SYNOPSIS

perl qle-<version_number>.pl [--<option>=<value>  [--<option>=<value> ...]]


=head1 DESCRIPTION

The 'qle'  program is a graphic  application for use  by amateur radio
operators.   The   logger  records   contacts  (QSOs)  in   an  SQLite
database.  It is  capable of  searching  for and  editing QSOs  logged
previously.

The logger  will poll a  connected transceiver for frequency  and mode
data   through   the   'rigctld'    server,   part   of   the   Hamlib
software. Real-time UTC date-  and timestamps are automatically logged
with the QSO. But QSOs can also be entered later.

The 'qle'  program is aimed  at rag-chewers, casual DX-ers  and casual
contesters (both  SSB and CW) and offers  on-the-fly dupe-checking and
serial  or fixed exchanges.  In addition,  'qle' can  transmit contest
exchanges  in morse  telegraphy (CW)  through the  optional 'cwdaemon'
server.

From the  'qle' program,  contest logs can  directly be dumped  into a
Cabrillo  file.  In  addition,  the 'qle'  distribution supports  ADIF
imports and  exports as well  as Cabrillo imports  through stand-alone
Perl scripts.

The SQLite log can be  accessed by third-party graphic front-ends such
as knoda,  or changed  and imported/exported to  SQL by  the 'sqlite3'
command-line utility.

The  config file contains  a large  number of  variables which  can be
tweaked to alter the appearance and behaviour of 'qle'.

It should  be possible to change  the number of QSO  fields - provided
the  program configuration is  in agreement  with the  actual database
schema. But  this process is so  far untested, because  the author has
not  had the  need to  make changes  to the  default schema  and field
labelling.

The full 'qle'  documentation is contained in the  file 'README', with
the exception  of the configuration variables, which  are described in
the commented config file.


=head2 OPTIONS

The  option=value  pairs on  the  command-line  override  (but do  not
interpolate    into   other)    variables   in    the    config   file
'qle-<version_number>.conf'

Please examine the config file  - which is extensively commented - for
further details.

Some of the more common options are:

=over 15

=item B<debug>

Can be '1' to produce verbose  debugging info on STDIN, or '0' to turn
it off.

=item B<db>

The  name of  the  SQLite  database file,  e.g.  the example  database
'foo3.db'

=item B<tableName>

The name of the database table that contains the log.

=item B<useRig>

If '0', the  program assumes no rig is connected and  it will not test
for the presence of one.

=item B<noCwDaemon>

If '0', CW support will not be tested for and will be turned off.

=item B<cwdWpm>

Start-up default keyer speed in WPM, e.g. '42'

=back

=head1 FILES

qle-<version_number>.conf

=head1 BUGS

See the file 'BUGS'.

=head1 AUTHOR

Wilbert Knol (ZL2BSJ)

knol00 /at/ gmail.com

=head1 SEE ALSO

L<cwdaemon(8)>, L<rigctl(1)>


=cut

